"""
======================COPYRIGHT/LICENSE START==========================

coordinatesIO.py: I/O for nmrStar coordinate saveframe

Copyright (C) 2005-2008 Wim Vranken (European Bioinformatics Institute)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

A copy of this license can be found in ../../../../license/LGPL.license

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)
- PDBe website (http://www.ebi.ac.uk/pdbe/)

- contact Wim Vranken (wim@ebi.ac.uk)
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
"""

import os

from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarFile
from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarGenericFile

from b2bTools.general.ccpn.format.general.Constants import defaultSeqInsertCode
from b2bTools.general.ccpn.format.general.Util import getSeqAndInsertCode

from b2bTools.general.ccpn.universal.Util import returnInt

#####################
# Class definitions #
#####################

class NmrStarFile(NmrStarFile):

  def initialize(self, version = '2.1.1'):

    self.coordinateFiles = []
    self.files = self.coordinateFiles

    if not self.version:
      self.version = version

    self.saveFrameName = 'conformer_family_coord_set'
    self.DataClassFile = NmrStarCoordinateFile

    self.components = [self.saveFrameName]
    self.setComponents()

  def read(self,verbose = 0):

    self.readComponent(verbose = verbose)

class NmrStarCoordinateFile(NmrStarGenericFile):

  def initialize(self,parent,saveFrame = None,maxModelNum = 999):

    self.saveFrame = saveFrame

    self.parent = parent
    self.version = parent.version

    self.chains = []
    self.hetGroups = []
    self.modelCoordinates = {}

    self.code = None
    self.details = None

    # Set maximum number of models to read. Won't work with project level reading...
    if not hasattr(self,'maxModelNum'):
      self.maxModelNum = maxModelNum

    self.ssBonds = []

    if self.saveFrame:
      self.parseSaveFrame()

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    #
    # Here do parsing and reading...
    #

    tableTagNames = {}
    chainIds = []

    if self.version == '2.1.1':

      return

    elif self.version == '3.0':

      tableName = '_Atom_site'
      tableTagNames['modelId'] = 'Model_ID'
      tableTagNames['Id'] = 'ID'
      tableTagNames['entityAssemblyId'] = 'Label_entity_assembly_ID'
      tableTagNames['entityId'] = 'Label_entity_ID'
      tableTagNames['indexId'] = 'Label_comp_index_ID'
      tableTagNames['compId'] = 'Label_comp_ID'
      tableTagNames['atomId'] = 'Label_atom_ID'
      tableTagNames['authorChainId'] = 'Auth_segment_code'
      tableTagNames['authorSeqCode'] = 'Auth_seq_ID'
      tableTagNames['authorResName'] = 'Auth_comp_ID'
      tableTagNames['authorAtomName'] = 'Auth_atom_ID'
      tableTagNames['atomType'] = 'Type_symbol'
      tableTagNames['x'] = 'Cartn_x'
      tableTagNames['y'] = 'Cartn_y'
      tableTagNames['z'] = 'Cartn_z'

    elif self.version == '3.1':

      tableName = '_Atom_site'
      tableTagNames['modelId'] = 'Model_ID'
      tableTagNames['Id'] = 'ID'
      tableTagNames['entityAssemblyId'] = 'Label_entity_assembly_ID'
      tableTagNames['entityId'] = 'Label_entity_ID'
      tableTagNames['indexId'] = 'Label_comp_index_ID'
      tableTagNames['compId'] = 'Label_comp_ID'
      tableTagNames['atomId'] = 'Label_atom_ID'
      tableTagNames['atomType'] = 'Type_symbol'
      tableTagNames['x'] = 'Cartn_x'
      tableTagNames['y'] = 'Cartn_y'
      tableTagNames['z'] = 'Cartn_z'
      tableTagNames['occupancy'] = 'Occupancy'
      tableTagNames['authorChainId'] = 'Auth_asym_ID'
      tableTagNames['authorSeqCode'] = 'Auth_seq_ID'
      tableTagNames['authorResName'] = 'Auth_comp_ID'
      tableTagNames['authorAtomName'] = 'Auth_atom_ID'
      tableTagNames['pdbChainId'] = 'PDB_strand_ID'
      tableTagNames['pdbSeqCode'] = 'PDB_residue_no'
      tableTagNames['pdbSeqInsertCode'] = 'PDB_ins_code'
      tableTagNames['pdbResName'] = 'PDB_residue_name'
      tableTagNames['pdbAtomName'] = 'PDB_atom_name'
      tableTagNames['pdbxChainId'] = 'PDBX_label_asym_ID'

    if self.saveFrame.tags.has_key('Details'):
      self.details = self.saveFrame.tags['Details']

    if self.saveFrame.tables.has_key(tableName):

      tableTags = self.saveFrame.tables[tableName].tags

      for i in range(0,len(tableTags[tableTagNames['Id']])):

        Id = tableTags[tableTagNames['entityAssemblyId']][i]
        modelId = tableTags[tableTagNames['modelId']][i]
        cifCode = tableTags[tableTagNames['compId']][i]
        indexId = tableTags[tableTagNames['indexId']][i]

        if modelId > self.maxModelNum:
          continue

        if not self.modelCoordinates.has_key(modelId):

          self.modelCoordinates[modelId] = []

        # Waters are handled differently - need to distinguish by sequence code as well.
        if cifCode == 'HOH':
          chainId =  (Id,indexId)
          isWaterChain = True
        else:
          chainId = Id
          isWaterChain = False

        if not chainId in chainIds:

          self.chains.append(NmrStarChain(Id,indexId, isWaterChain = isWaterChain))
          chainIds.append(chainId)

        self.modelCoordinates[modelId].append(NmrStarCoordinate())
        self.modelCoordinates[modelId][-1].setData(tableTags,i,tableTagNames)
        self.modelCoordinates[modelId][-1].setDefaultAttrs()

      tableName1 = '_Conformer_family_refinement'
      self.setStructMethods(tableName1)

      tableName2 = '_Conformer_family_software'
      self.setStructSoftwares(tableName2)

class NmrStarChain:

  def __init__(self,Id,firstIndexId,isWaterChain = False):

    self.chainId = Id
    self.firstIndexId = firstIndexId

    self.isWaterChain = isWaterChain

class NmrStarCoordinate:

  def __init__(self):

    self.Id = None
    self.modelId = None

    self.entityAssemblyId = None
    self.entityId = None
    self.indexId = None
    self.compId = None
    self.atomId = None

    self.atomName = None
    self.resName = None
    self.chainCode = None
    self.seqCode = None
    self.insertionCode = defaultSeqInsertCode

    ##################
    # All this stuff is handled separately in readCoordinates!!!
    self.authorChainId = None
    self.authorSeqCode = None
    self.authorResName = None
    self.authorAtomName = None
    ##################

    ##################
    # Necessary for PDB info transfer!!!
    self.pdbChainId = None
    self.pdbSeqCode = None
    self.pdbSeqInsertCode = None
    self.pdbResName = None
    self.pdbAtomName = None
    ##################

    self.origData = None

    self.x = None
    self.y = None
    self.z = None

    self.bFactor = None
    self.occupancy = 1.0
    self.atomType = None

  def setData(self,tableTags,i,tableTagNames):

    for attrName in tableTagNames.keys():
      if tableTags.has_key(tableTagNames[attrName]):
        setattr(self,attrName,tableTags[tableTagNames[attrName]][i])

  def setDefaultAttrs(self):

    self.atomName = self.atomId
    self.seqCode = self.indexId
    self.chainCode = self.entityAssemblyId
    self.resName = self.compId



import bz2
import pickle as pickle

class Pickle:

  def __init__(self):
    pass

  def dump_data(self, filename, data):
    # compression 9 is default
    with bz2.BZ2File(filename, 'wb') as f:
      pickle.dump(data, f, protocol=pickle.HIGHEST_PROTOCOL)

  def load_data(self, filename):
    with bz2.BZ2File(filename, 'rb') as f:
      data = pickle.load(f, encoding='latin1')
      return data
    return None




# Author: Robert Guthrie

#typ='dyna_back,psipred,ef,dyna_side'
#ws=3
#400 epoche 0.816002568073
import torch
import sys
import torch.autograd as autograd
import torch.nn as nn
import torch.optim as optim
from torch.nn import  BCELoss
from torch.nn import MaxPool1d
import numpy as np
torch.manual_seed(1)
def to_scalar(var):
    # returns a python float
    return var.view(-1).data.tolist()[0]


def argmax(vec):
    # return the argmax as a python int
    _, idx = torch.max(vec, 1)
    return to_scalar(idx)


def prepare_sequence(seq, to_ix):  #### faivet
    idxs = [to_ix[w] for w in seq]
    tensor = torch.LongTensor(idxs)
    return autograd.Variable(tensor)


# Compute log sum exp in a numerically stable way for the forward algorithm
def log_sum_exp(vec):
	max_score = vec[0, argmax(vec)]
	max_score_broadcast = max_score.view(1, -1).expand(1, vec.size()[1])
	return max_score + \
		torch.log(torch.sum(torch.exp(vec - max_score_broadcast)))
class BiLSTM_CRF(nn.Module):
	def __init__(self, hidden_dim,n_features,n_classes=2,cuda=True):
		self.classes_=[0,1]
		super(BiLSTM_CRF, self).__init__()
		self.hidden_dim = hidden_dim
		self.criterion = CrossEntropyLoss()#(torch.Tensor([1,0.35]))
		#self.vocab_size = vocab_size
		#self.tag_to_ix = tag_to_ix
		self.tagset_size = n_classes+2 ## begin e end
		self.STOP_TAG=3
		self.START_TAG=2
		self.tag_to_ix={}
		self.tag_to_ix[self.START_TAG]=2
		self.tag_to_ix[self.STOP_TAG]=3
		tag_to_ix=self.tag_to_ix
		self.cuda=cuda
		# self.word_embeds = nn.Embedding(vocab_size, embedding_dim)
		if self.cuda:
			self.lstm = nn.LSTM(n_features, hidden_dim // 2,num_layers=1, bidirectional=True).cuda()
			self.hidden2tag = nn.Linear(hidden_dim, self.tagset_size).cuda()
			self.transitions = nn.Parameter(torch.randn(self.tagset_size, self.tagset_size).cuda() )
		else:
			self.lstm = nn.LSTM(n_features, hidden_dim // 2,num_layers=1, bidirectional=True)
			self.hidden2tag = nn.Linear(hidden_dim, self.tagset_size)
			self.transitions = nn.Parameter(torch.randn(self.tagset_size, self.tagset_size))
		# Maps the output of the LSTM into tag space.

		# These two statements enforce the constraint that we never transfer
		# to the start tag and we never transfer from the stop tag
		self.transitions.data[tag_to_ix[self.START_TAG], :] = -10000
		self.transitions.data[:, tag_to_ix[self.STOP_TAG]] = -10000

		self.hidden = self.init_hidden()

	def init_hidden(self):
		if self.cuda:
			return (autograd.Variable(torch.randn(2, 1, self.hidden_dim // 2)).cuda(),autograd.Variable(torch.randn(2, 1, self.hidden_dim // 2)).cuda() )
		else:
			return (autograd.Variable(torch.randn(2, 1, self.hidden_dim // 2)),autograd.Variable(torch.randn(2, 1, self.hidden_dim // 2)) )

	def _forward_alg(self, feats):
		# Do the forward algorithm to compute the partition function
		if self.cuda:
			init_alphas = torch.Tensor(1, self.tagset_size).fill_(-10000.).cuda()
		else:
			init_alphas = torch.Tensor(1, self.tagset_size).fill_(-10000.)
		# START_TAG has all of the score.
		init_alphas[0][self.tag_to_ix[self.START_TAG]] = 0.

		# Wrap in a variable so that we will get automatic backprop
		forward_var = autograd.Variable(init_alphas)

		# Iterate through the sentence
		for feat in feats:
			alphas_t = []  # The forward variables at this timestep
			for next_tag in range(self.tagset_size):
				# broadcast the emission score: it is the same regardless of
				# the previous tag
				emit_score = feat[next_tag].view(
					1, -1).expand(1, self.tagset_size)
				# the ith entry of trans_score is the score of transitioning to
				# next_tag from i
				trans_score = self.transitions[next_tag].view(1, -1)
				# The ith entry of next_tag_var is the value for the
				# edge (i -> next_tag) before we do log-sum-exp
				next_tag_var = forward_var + trans_score + emit_score
				# The forward variable for this tag is log-sum-exp of all the
				# scores.
				alphas_t.append(log_sum_exp(next_tag_var))
			forward_var = torch.cat(alphas_t).view(1, -1)
		terminal_var = forward_var + self.transitions[self.tag_to_ix[self.STOP_TAG]]
		alpha = log_sum_exp(terminal_var)
		return alpha
	def get_params(self,deep=True):
		return {}
	def _get_lstm_features(self, sentence):

		self.hidden = self.init_hidden()
		sentence=sentence.view(len(sentence), 1, -1)
		#embeds = self.word_embeds(sentence).view(len(sentence), 1, -1)
		lstm_out, self.hidden = self.lstm(sentence, self.hidden)

		lstm_out = lstm_out.view(len(sentence), self.hidden_dim)
		lstm_feats = self.hidden2tag(lstm_out)
		return lstm_feats

	def _score_sentence(self, feats, tags):
		# Gives the score of a provided tag sequence
		if self.cuda:
			score = autograd.Variable(torch.Tensor([0]).cuda())
			tags = torch.cat([torch.LongTensor([self.tag_to_ix[self.START_TAG]]).cuda(), tags])
		else:
			score = autograd.Variable(torch.Tensor([0]))
			tags = torch.cat([torch.LongTensor([self.tag_to_ix[self.START_TAG]]), tags])
		#print torch.cat([torch.LongTensor([self.tag_to_ix[self.START_TAG]]).cuda(), tags])
		#raw_input()

		for i, feat in enumerate(feats):
			score = score + \
				self.transitions[tags[i + 1], tags[i]] + feat[tags[i + 1]]
		score = score + self.transitions[self.tag_to_ix[self.STOP_TAG], tags[-1]]
		return score

	def _viterbi_decode(self, feats):
		backpointers = []

		# Initialize the viterbi variables in log space
		init_vvars = torch.Tensor(1, self.tagset_size).fill_(-10000.)
		init_vvars[0][self.tag_to_ix[self.START_TAG]] = 0

		# forward_var at step i holds the viterbi variables for step i-1
		if  self.cuda:
			forward_var = autograd.Variable(init_vvars).cuda()
		else:
			forward_var = autograd.Variable(init_vvars)
		for feat in feats:
			bptrs_t = []  # holds the backpointers for this step
			viterbivars_t = []  # holds the viterbi variables for this step

			for next_tag in range(self.tagset_size):
				# next_tag_var[i] holds the viterbi variable for tag i at the
				# previous step, plus the score of transitioning
				# from tag i to next_tag.
				# We don't include the emission scores here because the max
				# does not depend on them (we add them in below)
				next_tag_var = forward_var + self.transitions[next_tag]
				best_tag_id = argmax(next_tag_var)
				bptrs_t.append(best_tag_id)
				viterbivars_t.append(next_tag_var[0][best_tag_id])
			# Now add in the emission scores, and assign forward_var to the set
			# of viterbi variables we just computed
			forward_var = (torch.cat(viterbivars_t) + feat).view(1, -1)
			backpointers.append(bptrs_t)

		# Transition to STOP_TAG
		terminal_var = forward_var + self.transitions[self.tag_to_ix[self.STOP_TAG]]
		best_tag_id = argmax(terminal_var)
		path_score = terminal_var[0][best_tag_id]

		# Follow the back pointers to decode the best path.
		best_path = [best_tag_id]
		for bptrs_t in reversed(backpointers):
			best_tag_id = bptrs_t[best_tag_id]
			best_path.append(best_tag_id)
		# Pop off the start tag (we dont want to return that to the caller)
		start = best_path.pop()
		assert start == self.tag_to_ix[self.START_TAG]  # Sanity check
		best_path.reverse()
		return path_score, best_path

	def neg_log_likelihood(self, sentence, tags):
		feats = self._get_lstm_features(sentence)
		forward_score = self._forward_alg(feats)
		gold_score = self._score_sentence(feats, tags)
		return forward_score - gold_score

	def forward(self, sentence):  # dont confuse this with _forward_alg above.
		# Get the emission scores from the BiLSTM
		lstm_feats = self._get_lstm_features(sentence)

		# Find the best path, given the features.
		score, tag_seq = self._viterbi_decode(lstm_feats)
		return score, tag_seq
class lstmCRF_pytorch():
	def __init__(self,HIDDEN_DIM = 10,epoche=10,lr=0.1, weight_decay=1e-4,cuda=True):
		self.START_TAG = -1
		self.STOP_TAG = -2
		self.HIDDEN_DIM = 4
		self.epoche=epoche
		self.lr=lr
		self.weight_decay=weight_decay
		self.cuda=cuda
	def fit(self,x,y,verbose=2,batch=None):
		if batch==None:
			self.batch=len(x) ### una volta sola
		else:
			self.batch=batch
		#x=[[[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0]],[[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0]]]
		#y=[[1,1,0,0,1,1,0,0,1,1],[0,0,1,1,0,0,1,1,0,0,1,1,0,0]]
		#training_data=[(x[0],y[0]),(x[1],y[1])]

		model = BiLSTM_CRF( self.HIDDEN_DIM,n_features=len(x[0][0]),cuda=self.cuda)
		#optimizer = optim.SGD(model.parameters(), lr=self.lr, weight_decay=self.weight_decay)
		optimizer = optim.Adam(model.parameters(), lr = 0.01)
		for epoch in range(self.epoche):
			if verbose>=2:
				print ('starting epoch',epoch)
			cont=0
			for i in range(len(x)):
				if verbose>=3:
					print ('\tstarting protein',i,'/',len(x))
				sentence, tags=(x[i],y[i])
				# Step 1. Remember that Pytorch accumulates gradients.
				# We need to clear them out before each instance

				# Step 2. Get our inputs ready for the network, that is,
				# turn them into Variables of word indices.
				if self.cuda:
					sentence_in = autograd.Variable(torch.Tensor(sentence)).cuda() #prepare_sequence(sentence, word_to_ix)
					targets = torch.LongTensor(tags).cuda()
				else:
					sentence_in = autograd.Variable(torch.Tensor(sentence)) #prepare_sequence(sentence, word_to_ix)
					targets = torch.LongTensor(tags)
				#print targets
				# Step 3. Run our forward pass.
				neg_log_likelihood = model.neg_log_likelihood(sentence_in, targets)

				#s+=float(neg_log_likelihood)
				# Step 4. Compute the loss, gradients, and update the parameters by
				# calling optimizer.step()
				neg_log_likelihood.backward()
				if self.batch==cont or cont==len(x)-2: #batch di 1==cont0
					#neg_log_likelihood.backward()
					optimizer.step()
					model.zero_grad()
					print ('\t',neg_log_likelihood)
				cont+=1
				#print
		# Check predictions after training
		self.model=model
		#precheck_sent = prepare_sequence(training_data[0][0], word_to_ix)
	def decision_function(self,x):
		y_pred=[]
		for i in x:
			if self.cuda:
				y_pred+=[list(self.model(autograd.Variable(torch.Tensor(i).cuda()))[1])]
			else:
				y_pred+=[list(self.model(autograd.Variable(torch.Tensor(i)))[1])]
		assert len(y_pred)==len(x)
		for i in range(len(y_pred)):
			assert len(y_pred[i])==len(x[i])
		return y_pred
		# We got it!
import torch
import torch.autograd as autograd
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
class LSTMTagger(nn.Module):

	def __init__(self, n_features,hidden_dim,cuda=False):
		super(LSTMTagger, self).__init__()
		self.hidden_dim = hidden_dim
		self.cuda=cuda
		# The LSTM takes word embeddings as inputs, and outputs hidden states
		# with dimensionality hidden_dim.
		if self.cuda:
			self.lstm = nn.LSTM(n_features, hidden_dim).cuda()
			self.lstm1 = nn.LSTM(hidden_dim, hidden_dim).cuda()
			self.lstm2 = nn.LSTM(hidden_dim, hidden_dim).cuda()
			self.lstm3 = nn.LSTM(hidden_dim, hidden_dim).cuda()
			self.sig=nn.Sigmoid()
			self.pooling=MaxPool1d(hidden_dim).cuda()
		else:
			self.lstm = nn.LSTM(n_features, hidden_dim)
			self.lstm1 = nn.LSTM(hidden_dim, hidden_dim)
			self.sig=nn.Sigmoid()
			self.pooling=MaxPool1d(hidden_dim)
			# The linear layer that maps from hidden state space to tag space
		self.hidden = self.init_hidden()
		self.hidden1 = self.init_hidden()
		self.hidden2 = self.init_hidden()
		self.hidden3 = self.init_hidden()
	def init_hidden(self):
		# Before we've done anything, we dont have any hidden state.
		# Refer to the Pytorch documentation to see exactly
		# why they have this dimensionality.
		# The axes semantics are (num_layers, minibatch_size, hidden_dim)
		if self.cuda:
			return (autograd.Variable(torch.zeros(1, 1, self.hidden_dim).cuda()),autograd.Variable(torch.zeros(1, 1, self.hidden_dim).cuda()))
		else:
			return (autograd.Variable(torch.zeros(1, 1, self.hidden_dim)),autograd.Variable(torch.zeros(1, 1, self.hidden_dim)))


	def forward(self, sentence):

		lstm_out, self.hidden = self.lstm(sentence.view(len(sentence), 1, -1), self.hidden)
		lstm_out, self.hidden=self.lstm1(lstm_out, self.hidden1)
		#lstm_out, self.hidden=self.lstm2(lstm_out, self.hidden2)
		#lstm_out, self.hidden=self.lstm3(lstm_out, self.hidden3)
		lstm_out=self.sig(lstm_out)

		tag_space=self.pooling(lstm_out)
		#tag_scores = F.log_softmax(tag_space)
		#print tag_space.view(len(sentence), 1)
		#raw_input()
		return tag_space.view(len(sentence),1)
class nn1(nn.Module):

	def __init__(self, n_features,hidden_dim=10,cuda=False):
		super(nn1, self).__init__()
		self.cuda=cuda
		if self.cuda:
			self.net = nn.Sequential(
								nn.Linear(n_features, hidden_dim),
								nn.ReLU(),
								#nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								#nn.BatchNorm1d(hidden_dim),
								#nn.Linear(hidden_dim, hidden_dim),
								#nn.ReLU(),
								#nn.BatchNorm1d(hidden_dim),
								#nn.Linear(hidden_dim, hidden_dim),
								#nn.ReLU(),
								#nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim,1),
								nn.Sigmoid()
								).cuda()
		else:
			self.net = nn.Sequential(nn.Linear(n_features, hidden_dim),
								nn.ReLU(),
								nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim,1).
								nn.Sigmoid()
								)
	def forward(self, sentence):
		#print sentence

		#print sentence.view(len(sentence),-1)

		out=self.net(sentence.view(len(sentence), -1))

		return out
class gru1(nn.Module):

	def __init__(self, n_features,hidden_dim=10,cuda=False):
		super(gru1, self).__init__()
		self.cuda=cuda
		self.hidden_dim=hidden_dim
		if self.cuda:
			self.net = nn.Sequential(
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								#nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								#nn.BatchNorm1d(hidden_dim),
								#nn.Linear(hidden_dim, hidden_dim),
								#nn.ReLU(),
								#nn.BatchNorm1d(hidden_dim),
								#nn.Linear(hidden_dim, hidden_dim),
								#nn.ReLU(),
								#nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim,1),
								nn.Sigmoid()
								).cuda()



		else:
			self.net = nn.Sequential(nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim, hidden_dim),
								nn.ReLU(),
								nn.BatchNorm1d(hidden_dim),
								nn.Linear(hidden_dim,1).
								nn.Sigmoid()
								)
		self.hidden = self.init_hidden()
		self.gru=nn.GRU(n_features,hidden_dim,bidirectional=True).cuda()
		self.pooling=nn.MaxPool1d(2)
	def init_hidden(self):
		if self.cuda:
			return autograd.Variable(torch.randn(2, 1, self.hidden_dim)).cuda().float()
		else:
			return (autograd.Variable(torch.randn(2, 1, self.hidden_dim )),autograd.Variable(torch.randn(2, 1, self.hidden_dim // 2)) )

	def forward(self, sentence):
		#print sentence
		out=[]
		#print sentence.view(len(sentence),-1)
		for sen in sentence:
			a= sen.view(len(sen), 1, -1).float()
			#print a
			#print self.hidden
			gru_out,hidden=self.gru(a, self.hidden)
			#print gru_out
			gru_out=self.pooling(gru_out)
			#print gru_out
			#raw_input()
			gru_out.view(len(gru_out), -1)
			out+=[self.net(gru_out.view(len(gru_out), -1))]
		return out
class lstm_pytorch():
	def __init__(self,HIDDEN_DIM = 10,epoche=10,lr=0.1, weight_decay=1e-4,cuda=False):
		self.epoche=epoche
		self.lr=lr
		self.weight_decay=weight_decay
		self.cuda=cuda
		self.HIDDEN_DIM = 4
	def fit(self,x,y,verbose=2,batch=None):
		if batch==None:
			self.batch=len(x) ### una volta sola
		else:
			self.batch=batch
		model = LSTMTagger(n_features=len(x[0][0]), hidden_dim=self.HIDDEN_DIM,cuda=self.cuda)
		#optimizer = optim.SGD(model.parameters(), lr=self.lr, weight_decay=self.weight_decay)
		optimizer = optim.Adam(model.parameters(), lr = 0.5)
		loss_function = BCELoss()

		p=[]
		for i in  model.parameters():
			p+= list(i.data.cpu().numpy().flat)
		print ('number of parameters=',len(p))

		for epoch in range(self.epoche):
			if verbose>=2:
				sys.stdout.write('starting epoch'+str(epoch)+' ')

			loss_tot=[]
			cont=0
			model.zero_grad()
			for i in range(len(x)):
				if verbose>=3:
					print ('\tstarting protein',i,'/',len(x))
				sentence, tags=(x[i],y[i])
				if self.cuda:
					sentence_in = autograd.Variable(torch.Tensor(sentence)).cuda() #prepare_sequence(sentence, word_to_ix)
					targets = autograd.Variable(torch.FloatTensor(tags)) .cuda()
				else:
					sentence_in = autograd.Variable(torch.Tensor(sentence)) #prepare_sequence(sentence, word_to_ix)
					targets =  autograd.Variable(torch.FloatTensor(tags))
				tag_scores = model(sentence_in)

				model.hidden = model.init_hidden()

				targets=targets.view(len(targets),1)

				loss = loss_function(tag_scores, targets.view(len(targets),-1))
				loss.backward()
				loss_tot+=[float(loss.cpu().data.numpy().flat[0])]

				#print loss

			optimizer.step()
			model.zero_grad()
			sys.stdout.write( ' '+str(np.mean(loss_tot))+'\n')

		self.model=model
		#precheck_sent = prepare_sequence(training_data[0][0], word_to_ix)
	def decision_function(self,x):
		y_pred=[]
		for i in x:
			if self.cuda:
				a=list(self.model(autograd.Variable(torch.Tensor(i).cuda())).data.cpu().numpy().flat)

				y_pred+=[a]
			else:
				a=list(self.model(autograd.Variable(torch.Tensor(i))).data.numpy().flat)

				y_pred+=[a]
		assert len(y_pred)==len(x)

		for i in range(len(y_pred)):

			assert len(y_pred[i])==len(x[i])
		return y_pred
		# We got it!
class nn_pytorch():
	def __init__(self,HIDDEN_DIM = 30,epoche=100,lr=0.001, weight_decay=1e-4,batch=5,cuda=True,name=None):
		self.epoche=epoche
		self.lr=lr
		self.weight_decay=weight_decay
		self.cuda=cuda
		self.HIDDEN_DIM=HIDDEN_DIM
		self.batch=batch
		self.name=name
	def fit(self,x,y,verbose=2,batch=None):
		if batch==None:
			self.batch=len(x) ### una volta sola
		else:
			self.batch=batch
		model = gru1(n_features=len(x[0][0]), hidden_dim=self.HIDDEN_DIM,cuda=self.cuda)
		#optimizer = optim.SGD(model.parameters(), lr=self.lr, weight_decay=self.weight_decay)
		optimizer = optim.Adam(model.parameters())
		loss_function = BCELoss()

		p=[]
		for i in  model.parameters():
			p+= list(i.data.cpu().numpy().flat)
		print ('number of parameters=',len(p))
		cont=1
		verbose=2
		for epoch in range(self.epoche):
			model.zero_grad()
			sentence_in=[]
			if verbose>=2:
				sys.stdout.write('starting epoch'+str(epoch)+' ')

			loss_tot=[]

			model.zero_grad()

			if verbose>=3:
				print ('\tstarting protein',i,'/',len(x))
			sentence, tags=(x,y)
			targets=[]
			if self.cuda:
				for i in sentence:
					sentence_in += [autograd.Variable(torch.Tensor(i)).cuda()] #prepare_sequence(sentence, word_to_ix)
				for i in tags:
					targets += [autograd.Variable(torch.FloatTensor(i)).cuda()]
			else:
				sentence_in = autograd.Variable(torch.Tensor(sentence)) #prepare_sequence(sentence, word_to_ix)
				targets =  autograd.Variable(torch.FloatTensor(tags))
			tag_scores = model(sentence_in)
			a=[]
			for i in range(len(tag_scores)):
				true=targets[i].view(len(targets[i]),1)
				a+=[loss_function(tag_scores[i], true)]

			loss =torch.mean(torch.stack(a))

			loss_tot+=[float(loss.cpu().data.numpy().flat[0])]
			cont+=1

			loss.backward()
			optimizer.step()
			sys.stdout.write( ' '+str(np.mean(loss_tot))+'\n')
			if epoch%20==0 and epoch !=0:
				if self.name!= None:
					torch.save(model,'/home/scimmiacasa/Dropbox/modelli_disordine/'+self.name+'_gru80_'+str(epoch)+'_epochs.mtorch')
				else:
					torch.save(model,'/home/scimmiacasa/Dropbox/modelli_disordine/gru80_'+str(epoch)+'_epochs.mtorch')
		self.model=model
		#precheck_sent = prepare_sequence(training_data[0][0], word_to_ix)
		torch.save(model.cpu(),'gru80_final.mtorch')
	def load_model(self,fil):
		self.model=torch.load(fil,map_location={'cuda:0': 'cpu'})
	def decision_function(self,x):
		y_pred=[]
		#self.model=torch.load('gru80_final.mtorch',map_location={'cuda:0': 'cpu'})
		for i in x:
			a=list(self.model([autograd.Variable(torch.Tensor(i).float())])[0].data.cpu().numpy().flat)
			y_pred+=[a]
		assert len(y_pred)==len(x)

		for i in range(len(y_pred)):

			assert len(y_pred[i])==len(x[i])
		return y_pred
if __name__ == '__main__':
		x=[[[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0]],[[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0]]]
		y=[[1,1,0,0,1,1,0,0,1,1],[0,0,1,1,0,0,1,1,0,0,1,1,0,0]]
		a=nn_pytorch(epoche=30,cuda=True)
		#a.fit(x,y,batch=1)
		print (a.decision_function([[[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0],[1,1,1],[0,0,0],[1,0,1],[1,0,0]]]))



from .singleSeq.DynaMine.Predictor import DynaMine
from .singleSeq.EFoldMine.Predictor import EFoldMine
from .singleSeq.DisoMine.Predictor import DisoMine
import numpy as np
import json

#
# @Luciano: This script might be better incorporated on the web end itself, also for error catching!
#

def predictSeqs(seqs, extra_predictions):

    """
    :param seqs:
    :param extra_predictions:
    :return:
    """

    # NOTE Wim: this has to change, only run what is required! Need to discuss how to do the bookkeeping here...


    dm = DynaMine()
    dm.predictSeqs(seqs)
    efm = EFoldMine()
    efm.predictSeqs(seqs)
    de = DisoMine()
    de.allPredictions = efm.allPredictions
    de.predictSeqs(seqs)

    out=[]
    for protID in range(len(seqs)):
        entry={}
        entry['proteinID']=json.loads(de.getAllPredictionsJson('Disomine'))['results'][protID]['proteinID']
        entry['sequence']=json.loads(de.getAllPredictionsJson('Disomine'))['results'][protID]['sequence']
        entry['disomine']=np.array(json.loads(de.getAllPredictionsJson('Disomine'))['results'][protID]['disoMine'])

        if extra_predictions:
            entry['efoldmine']=np.array(json.loads(de.getAllPredictionsJson('Disomine'))['results'][protID]['earlyFolding'])
            entry['backbone']=np.array(json.loads(de.getAllPredictionsJson('Disomine'))['results'][protID]['backbone'])
            entry['sidechain']=np.array(json.loads(de.getAllPredictionsJson('Disomine'))['results'][protID]['sidechain'])
            assert len(entry['sequence'])==len(entry['disomine'])==len(entry['efoldmine'])==len(entry['sidechain'])==len(entry['backbone'])
        out+=[entry]

    return {'results':out}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  phase_transition_hmm.py
#
#  Copyright 2018 scimmia <scimmia@scimmia-ThinkPad-L540>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
from sklearn.metrics import roc_auc_score
from scipy.stats import ranksums
import matplotlib.pyplot as plt
from b2bTools.singleSeq.PSPer.hmmer_research.hmmsearch_otf import hmmer_wrapper
from b2bTools.singleSeq.PSPer.source.utils import seqs_features,leggifasta
import numpy as np
import pickle, string, random
from pomegranate import *
from scipy.optimize import minimize
import json
import os, pathlib
from b2bTools.singleSeq.DynaMine.Predictor import DynaMine
from b2bTools.singleSeq.EFoldMine.Predictor import EFoldMine
from b2bTools.singleSeq.DisoMine.Predictor import DisoMine

def load_pickle(pickle_file):
    try:
        with open(pickle_file, 'rb') as f:
            pickle_data = pickle.load(f)
    except UnicodeDecodeError as e:
        with open(pickle_file, 'rb') as f:
            pickle_data = pickle.load(f, encoding='latin1')
    except Exception as e:
        print('Unable to load data ', pickle_file, ':', e)
        raise
    return pickle_data

class phase_hmm:

	def __init__(self):
		self.emiss_diverse=None
		self.emiss_bg=None
		self.scaler=None

		self.scriptDir = str(pathlib.Path(__file__).parent.absolute())

	def fit(self):
		if 'emissions.m' in os.listdir(os.path.join(self.scriptDir,'marshalled')):
			print ('loading emissions')
			self.emiss_diverse,self.emiss_bg=load_pickle(os.path.join(self.scriptDir,'marshalled/emissions.m'))
		else:
			print ('calculating emissions')
			self.emiss_diverse,self.emiss_bg=self.learn_distro()
			pickle.dump((self.emiss_diverse,self.emiss_bg),open(os.path.join(self.scriptDir,'marshalled/emissions.m','w')))

		self.build_hmm()

		if 'scaler_new.m' in os.listdir(os.path.join(self.scriptDir,'marshalled')):
			print ('loading scaler')
			self.scaler=load_pickle(os.path.join(self.scriptDir,'marshalled/scaler_new.m'))
		else:
			print ('calculating scaler')
			self.scaler=self.learn_scaling()
			pickle.dump(self.scaler,open(os.path.join(self.scriptDir,'marshalled/scaler_new.m','w')))


	def build_hmm(self,param=None,optim_params=False,PLOT_DISTROS=False):
		if self.emiss_diverse==None or self.emiss_bg==None:
			raise ValueError('YOU FIRST NEED TO FIT EMISSIONS! RUN .fit()')
		diverso=self.emiss_diverse
		costante=self.emiss_bg
		if not optim_params:
			len_pld=80
			len_rna=50
			len_altro=20
			len_grigio=15

			### param transizioni ###
			TRANS_ALTRO=0.001

			## somma a 1 ###
			TRANS_ESTENDI_STATO=0.6

			################

		else:
			len_pld=int(param[1])
			len_rna=int(param[2])
			len_altro=int(param[3])
			len_grigio=int(param[4])
		diz_rna={}
		diz_NON_rna={}

		for i in range(11):
			if i==10:
				diz_rna['a']=0.05+i*0.1*0.9/5.5
			else:

				diz_rna[str(i)]=0.05+i*0.1*0.9
		for i in range(10,-1,-1):
			if i!=10:
				diz_NON_rna[str(i)]=0.05+(10-i)*0.1*0.9
			else:

				diz_NON_rna['a']=0.05+(10-i)*0.1*0.9
		norm=sum(diz_rna.values())
		for i in diz_rna.keys():
			diz_rna[i]=diz_rna[i]/norm
		norm=sum(diz_NON_rna.values())
		for i in diz_NON_rna.keys():
			diz_NON_rna[i]=diz_NON_rna[i]/norm
		rna_alto=DiscreteDistribution(diz_rna)
		rna_basso=DiscreteDistribution(diz_NON_rna)
		#rna_alto=DiscreteDistribution(diverso[3])
		#rna_basso=DiscreteDistribution(costante[4])
		#print diz_NON_rna
		#ad
		if PLOT_DISTROS:
			features=['complexity','G-R_enrichment','Y_enrichment','RNA','Disorder']
			for i in range(len(features)):
				data_c = costante[i]
				data_d =diverso[i]
				cos=[]
				div=[]
				for k in sorted(data_c.keys()):
					cos+=[data_c[k]]
					div+=[data_d[k]]
				if features[i]!='complexity':
					plt.plot(range(len(cos)),cos,'r',label='background distr.')
					plt.plot(range(len(cos)),div,'b',label='inferred PSP distr.')
				else:
					plt.plot(range(len(cos)),div,'b',label='inferred PSP distr.')
					plt.plot(range(len(cos)),cos,'r',label='background distr.')
				plt.xlabel(features[i])
				plt.ylabel('Emission probability')
				plt.legend()
				#left_of_first_bin = 0
				#right_of_last_bin = len(data_k.keys())#data.max() + float(d)/2
				#plt.hist(data, np.arange(left_of_first_bin, right_of_last_bin + d, d))
				plt.title(features[i])
				plt.savefig('plot_distros/'+features[i]+'.png')
				plt.clf()



		distrib_altro =IndependentComponentsDistribution([DiscreteDistribution(costante[0]),DiscreteDistribution(costante[1]),DiscreteDistribution(costante[2]),rna_basso,DiscreteDistribution(costante[4])])
		distrib_grigia=IndependentComponentsDistribution([DiscreteDistribution(costante[0]),DiscreteDistribution(diverso[1]),DiscreteDistribution(costante[2]),rna_basso,DiscreteDistribution(diverso[4])])
		distrib_rna   =IndependentComponentsDistribution([DiscreteDistribution(costante[0]),DiscreteDistribution(costante[1]),DiscreteDistribution(costante[2]),rna_alto,DiscreteDistribution(costante[4])])
		distrib_pdl   =IndependentComponentsDistribution([DiscreteDistribution(diverso[0]),DiscreteDistribution(costante[1]),DiscreteDistribution(diverso[2]),rna_basso,DiscreteDistribution(diverso[4])])
		PLD1=[]
		PLD2=[]
		altroB=[]
		altroF=[]

		altroRNA=[]
		altroPLD=[]
		grigioB=[]
		grigioF=[]
		grigio_intra1=[]
		grigio_intra2=[]
		grigioRNA=[]
		grigioPLD=[]
		RNA1=[]
		RNA2=[]

		for i in range(len_pld):
			PLD1+=[State(distrib_pdl,name='PLD_1'+str(i))]
			PLD2+=[State(distrib_pdl,name='PLD_2'+str(i))]
		for i in range(len_rna):
			RNA1+=[State(distrib_rna,name='RRM_1'+str(i))]
			RNA2+=[State(distrib_rna,name='RRM_2'+str(i))]
		for i in range(len_altro):
			altroB+=[State(distrib_altro,name='OTHER_B_'+str(i))]
			altroPLD+=[State(distrib_altro,name='OTHER_PLD_'+str(i))]
			altroRNA=[State(distrib_altro,name='OTHER_RNA_'+str(i))]
			altroF+=[State(distrib_altro,name='OTHER_F_'+str(i))]
		for i in range(len_grigio):
			grigioB+=[State(distrib_grigia,name='SPACER_B_'+str(i))]
			grigioF+=[State(distrib_grigia,name='SPACER_F_'+str(i))]
			grigioPLD+=[State(distrib_grigia,name='SPACER_PLD_'+str(i))]
			grigioRNA+=[State(distrib_grigia,name='SPACER_RNA_'+str(i))]
			grigio_intra1+=[State(distrib_grigia,name='SPACER_intra2_'+str(i))]
			grigio_intra2+=[State(distrib_grigia,name='SPACER_intra1_'+str(i))]
		model = HiddenMarkovModel()
		background= HiddenMarkovModel()
		bg=State(distrib_altro,name='background')
		background.add_states(bg)
		background.add_transition(bg ,bg ,0.7)
		background.add_transition(bg ,background.end ,0.3)
		background.add_transition(background.start,bg,1)
		background.bake()
		self.background=background
		# grigio
		for i in range(len(grigioB)):
			model.add_states(grigioB[i])
		for i in range(len(grigioF)):
			model.add_states(grigioF[i])
		for i in range(len(grigioRNA)):
			model.add_states(grigioRNA[i])
		for i in range(len(grigioPLD)):
			model.add_states(grigioPLD[i])
		for i in range(len(grigio_intra1)):
			model.add_states(grigio_intra1[i])
		for i in range(len(grigio_intra2)):
			model.add_states(grigio_intra2[i])
		# altro
		for i in range(len(altroB)):
			model.add_states(altroB[i])
		for i in range(len(altroF)):
			model.add_states(altroF[i])
		for i in range(len(altroPLD)):
			model.add_states(altroPLD[i])
		for i in range(len(altroRNA)):
			model.add_states(altroRNA[i])
		#######################################
		#PLD
		for i in range(len(PLD1)):
			model.add_states(PLD1[i])
		for i in range(len(PLD2)):
			model.add_states(PLD2[i])
		#RNA
		for i in range(len(RNA1)):
			model.add_states(RNA1[i])
		for i in range(len(RNA2)):
			model.add_states(RNA2[i])
		#######################################
		#              GRIGIO
		for i in range(len(grigioB)-1):
			model.add_transition(grigioB[i],grigioB[i+1] , 1)
		model.add_transition(grigioB[-1],grigioB[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(grigioF)-1):
			model.add_transition(grigioF[i],grigioF[i+1] , 1)
		model.add_transition(grigioF[-1],grigioF[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(grigioPLD)-1):
			model.add_transition(grigioPLD[i],grigioPLD[i+1] , 1)
		model.add_transition(grigioPLD[-1],grigioPLD[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(grigioRNA)-1):
			model.add_transition(grigioRNA[i],grigioRNA[i+1] , 1)
		model.add_transition(grigioRNA[-1],grigioRNA[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(grigio_intra1)-1):
			model.add_transition(grigio_intra1[i],grigio_intra1[i+1] , 1)
		model.add_transition(grigio_intra1[-1],grigio_intra1[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(grigio_intra2)-1):
			model.add_transition(grigio_intra2[i],grigio_intra2[i+1] , 1)
		model.add_transition(grigio_intra2[-1],grigio_intra2[-1] , TRANS_ESTENDI_STATO)

		################################################
		#                 altro
		for i in range(len(altroB)-1):
			model.add_transition(altroB[i],altroB[i+1] , 1)
		model.add_transition(altroB[-1],altroB[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(altroPLD)-1):
			model.add_transition(altroPLD[i],altroPLD[i+1] , 1)
		model.add_transition(altroPLD[-1],altroPLD[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(altroF)-1):
			model.add_transition(altroF[i],altroF[i+1] , 1)
		model.add_transition(altroF[-1],altroF[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(altroRNA)-1):
			model.add_transition(altroRNA[i],altroRNA[i+1] , 1)
		model.add_transition(altroRNA[-1],altroRNA[-1] , TRANS_ESTENDI_STATO)

		################################################
		#                  rna
		for i in range(len(RNA1)-1):
			model.add_transition(RNA1[i],RNA1[i+1] , 1)
		model.add_transition(RNA1[-1],RNA1[-1] , TRANS_ESTENDI_STATO)
		for i in range(len(RNA2)-1):
			model.add_transition(RNA2[i],RNA2[i+1] , 1)
		model.add_transition(RNA2[-1],RNA2[-1] , TRANS_ESTENDI_STATO)

		################################################
		#                 pld
		for i in range(len(PLD1)-1):
			model.add_transition(PLD1[i],PLD1[i+1] , 1)
		model.add_transition(PLD1[-1],PLD1[-1] , TRANS_ESTENDI_STATO)

		for i in range(len(PLD2)-1):
			model.add_transition(PLD2[i],PLD2[i+1] , 1)
		model.add_transition(PLD2[-1],PLD2[-1] , TRANS_ESTENDI_STATO)

		##################### MODULO 1 #######################
		#  somma a 0.05 per avere estensione di 20 al blocco #
		# la P disponibile e' 1-P(self-transizione interna al blocco) --> ora a 0.95 --> estensione media 20
		######################################################


		######################################################à
		#                    BEGIN                            #
		#######################################################

		model.add_transition(model.start,PLD1[0] , 0.5)
		model.add_transition(model.start,grigioB[0] , 0.3)
		model.add_transition(model.start,altroB[0] , TRANS_ALTRO)
		model.add_transition(model.start,RNA1[0] , 0.1)
		######################################################à
		#                    altrob                           #
		#######################################################
		model.add_transition(altroB[-1],PLD1[0] , 0.1*(1-TRANS_ESTENDI_STATO))
		model.add_transition(altroB[-1],grigioB[0] , 0.8*(1-TRANS_ESTENDI_STATO))
		model.add_transition(altroB[-1],RNA1[0] , 0.1*(1-TRANS_ESTENDI_STATO))

		######################################################à
		#                    grigioB                            #
		#######################################################
		model.add_transition(grigioB[-1],PLD1[0] , 0.45*(1-TRANS_ESTENDI_STATO))
		model.add_transition(grigioB[-1],altroB[0] , TRANS_ALTRO*(1-TRANS_ESTENDI_STATO))
		model.add_transition(grigioB[-1],RNA1[0] , 0.45*(1-TRANS_ESTENDI_STATO))

		######################################################à
		#                    RNA1                            #
		#######################################################
		model.add_transition(RNA1[-1],PLD2[0] , 0.1*(1-TRANS_ESTENDI_STATO))
		model.add_transition(RNA1[-1],grigio_intra1[0] , 0.6*(1-TRANS_ESTENDI_STATO))
		model.add_transition(RNA1[-1],grigioRNA[0] , 0.4*(1-TRANS_ESTENDI_STATO))
		model.add_transition(RNA1[-1],altroRNA[0] , TRANS_ALTRO*(1-TRANS_ESTENDI_STATO))

		model.add_transition(grigio_intra1[-1],RNA1[0] , 1*(1-TRANS_ESTENDI_STATO))

		######################################################à
		#                    grigioRNA                        #
		#######################################################
		model.add_transition(grigioRNA[-1],PLD2[0] , 0.8*(1-TRANS_ESTENDI_STATO))
		model.add_transition(grigioRNA[-1],altroRNA[0] , TRANS_ALTRO*(1-TRANS_ESTENDI_STATO))


		######################################################à
		#                    altroRNA                        #
		#######################################################
		model.add_transition(altroRNA[-1],PLD2[0] , 0.1*(1-TRANS_ESTENDI_STATO))
		model.add_transition(altroRNA[-1],grigioRNA[0] , 0.9*(1-TRANS_ESTENDI_STATO))


		######################################################à
		#                   PLD2                             #
		#######################################################
		model.add_transition(PLD2[-1],altroF[0] ,TRANS_ALTRO*(1-TRANS_ESTENDI_STATO))
		model.add_transition(PLD2[-1],grigioF[0] , 0.1*(1-TRANS_ESTENDI_STATO))
		model.add_transition(PLD2[-1],model.end ,0.8*(1-TRANS_ESTENDI_STATO))

		######################################################à
		#                    altroF                      #
		#######################################################
		model.add_transition(altroF[-1],model.end , 0.2*(1-TRANS_ESTENDI_STATO))
		model.add_transition(altroF[-1],grigioF[0] , 0.8*(1-TRANS_ESTENDI_STATO))

		######################################################à
		#                    grigioF                      #
		#######################################################
		model.add_transition(grigioF[-1],model.end , 0.2*(1-TRANS_ESTENDI_STATO))
		model.add_transition(grigioF[-1],altroF[0] , 0.8*(1-TRANS_ESTENDI_STATO))

		######################################################à
		#                   PLD1                             #
		#######################################################
		model.add_transition(PLD1[-1],altroPLD[0] , TRANS_ALTRO*(1-TRANS_ESTENDI_STATO))
		model.add_transition(PLD1[-1],grigioPLD[0] , 0.8*(1-TRANS_ESTENDI_STATO))
		model.add_transition(PLD1[-1],RNA2[0] ,0.1*(1-TRANS_ESTENDI_STATO))

		######################################################à
		#                    grigioPLD                        #
		#######################################################
		model.add_transition(grigioPLD[-1],RNA2[0] , 0.8*(1-TRANS_ESTENDI_STATO))
		model.add_transition(grigioPLD[-1],altroPLD[0] , TRANS_ALTRO*(1-TRANS_ESTENDI_STATO))


		######################################################à
		#                    altroPLD                       #
		#######################################################
		model.add_transition(altroPLD[-1],RNA2[0] , 0.2*(1-TRANS_ESTENDI_STATO))
		model.add_transition(altroPLD[-1],grigioPLD[0] , 0.8*(1-TRANS_ESTENDI_STATO))

		######################################################à
		#                    RNA2                            #
		#######################################################
		model.add_transition(RNA2[-1],grigio_intra2[0] , 0.4*(1-TRANS_ESTENDI_STATO))
		model.add_transition(RNA2[-1],grigioF[0] , 0.4*(1-TRANS_ESTENDI_STATO))
		model.add_transition(RNA2[-1],altroF[0] , TRANS_ALTRO*(1-TRANS_ESTENDI_STATO))
		model.add_transition(RNA2[-1],model.end , 0.2*(1-TRANS_ESTENDI_STATO))
		model.add_transition(grigio_intra2[-1],RNA2[0] , 1*(1-TRANS_ESTENDI_STATO))
		######

		model.bake()

		self.model=model
		#import json
		#a=self.model.to_json()
		#open('params.json','w').write(a)

	def build_vector(self, seqs):
		SEQS = []
		for key, value in seqs.items():
			temp = (key, value)
			SEQS.append(temp)


		# TODO - could still improve this by providing the predictions if already run prior to PSPer request! Will speed things up...
		dm = DynaMine(predictionTypes=['backbone', 'sidechain'])
		dm.predictSeqs(SEQS)
		efm = EFoldMine()
		efm.allPredictions = dm.allPredictions
		efm.predictSeqs(SEQS)
		de = DisoMine()
		de.allPredictions = efm.allPredictions
		disomine=de.predictSeqs(SEQS)
		entry={}
		for protID in range(len(seqs)):
			entry[json.loads(de.getAllPredictionsJson('Disomine'))['results'][protID]['proteinID']] = np.array(json.loads(de.getAllPredictionsJson('Disomine'))['results'][protID]['disoMine'])

		disomine = entry
		hmmer_wrapper_instance = hmmer_wrapper(root=os.path.join(self.scriptDir, 'hmmer_research/'))
		hmmer_results = hmmer_wrapper_instance.predict(seqs)

		hmmer_features = seqs_features(seqs)
		result_dict = {}

		for hmmer_key in hmmer_results.keys():
			current_hmmer_result = hmmer_results[hmmer_key]
			current_features = hmmer_features[hmmer_key]
			current_disorder = disomine[hmmer_key]

			result_dict[hmmer_key] = discretizza([
				current_features[0],
				current_features[1],
				current_features[2],
				current_hmmer_result,
				current_disorder
			])

		return result_dict

	def predict_proba(self,seq,scale=True):

		if type(seq)==str:
			seq={'input_seq':seq}
			vets=self.build_vector(seq)
		elif type(seq)==dict and type(seq[list(seq.keys())[0]])==str:
			vets=self.build_vector(seq)
		elif type(seq)==dict and type(seq[list(seq.keys())[0]])==list:
			vets = seq
		else:
			raise TypeError('predict takes a protein sequnce (str) or a dict of sequences or a dict of feature vectors (list)')
		res={}

		for i in vets.keys():
			true=self.model.log_probability(vets[i])
			rand=self.background.log_probability(vets[i])
			score=true-rand

			if scale==True:
				cattivo=False

				if score<-10000:
					score=-10000
					cattivo=True
				score=self.scaler.transform([[score]])[0][0]
				if cattivo:
					score=0

				if score<0:
					score=0.0
				elif score>1:
					score=1.0
			res[i]=float(score)
		return res

	def viterbi(self,seq,scale=False):
		if type(seq)==str:
			seq={'input_seq':seq}
			vet=self.build_vector(seq)
		elif type(seq)==dict and type(seq[list(seq.keys())[0]])==str:
			vet=self.build_vector(seq)
		elif type(seq)==dict and type(seq[list(seq.keys())[0]])==list:
			vet = seq
		else:
			raise TypeError('predict takes a protein sequnce (str) or a dict of sequences or a dict of feature vectors (list)')
		r={}
		for i in seq.keys():
			p=[]
			seq_key = vet[i]
			viterbi_model = self.model.viterbi(seq_key)[1]
			if not viterbi_model:
				print(f"No Viterbi model for input = {seq_key}")
				continue
			else:
				path= ", ".join(state.name for i, state in viterbi_model)
				for k in path.split(',')[1:-1]:
					p+=[k.split('_')[0]]

				assert len(p)==len(seq[i])

				r[i]=p

		return r

	def learn_scaling(self):
		from sklearn.preprocessing import MinMaxScaler

		if FAST and 'backgroundDataset.fasta.cPickle' in os.listdir('marshalled'):
			vets_bg=load_pickle('marshalled/backgroundDataset.fasta.cPickle')

		else:
			print ('no vect marshal for background, this will take ages')
			seqs_bg=leggifasta('validationDatasets/background/backgroundDataset.fasta')
			s={}

			for i in seqs_bg.keys()[:]:
				if len(seqs_bg[i])>=MAX_SEQ_SIZE or len(seqs_bg[i])<MIN_SEQ_SIZE:
					continue
				s[i]=seqs_bg[i]
			seqs_bg=s
			vets_bg=self.build_vector(seqs_bg)
			pickle.dump((vets_bg,seqs_bg),open('marshalled/backgroundDataset.fasta.cPickle','w'))

		if "phaseSepVects.cPickle" in os.listdir('marshalled'):
			vets=load_pickle('marshalled/phaseSepVects.cPickle')
		else:
			seqs=leggifasta('validationDatasets/phaseSepProts/phase_separation_proteins.fasta')
			print ('no vect marshal for phase_separation, marshalling.',len(seqs),'proteins to do')
			s={}
			for i in seqs.keys()[:]:

				if len(seqs[i])>=MAX_SEQ_SIZE:
					continue
				s[i]=seqs[i]
			seqs=s
			vets=self.build_vector(seqs)
			pickle.dump(vets,open('marshalled/phaseSepVects.cPickle','w'))


		if "RNABPdatasetFinal.fasta.cPickle" in os.listdir('marshalled'):
			vetsbp=load_pickle('marshalled/RNABPdatasetFinal.fasta.cPickle')
			#del vetsbp['Q9UQ35']
			#pickle.dump(vetsbp,open('marshalled/RNABPdatasetFinal.fasta.cPickle','w'))
		else:
			seqsbp=leggifasta('validationDatasets/RNAbp/RNABPdatasetFinal.fasta')
			print ('no vect marshal for phase_separation, marshalling.',len(seqsbp),'proteins to do')
			s={}
			for i in seqsbp.keys()[:]:

				if len(seqsbp[i])>=MAX_SEQ_SIZE:
					continue
				s[i]=seqsbp[i]
			seqsbp=s
			vetsbp=self.build_vector(seqsbp)


			pickle.dump(vetsbp,open('marshalled/RNABPdatasetFinal.fasta.cPickle','w'))
		#vets.update(vets_bg)
		#print vetsbp[0]
		#print vets
		vetsbp.update(vets_bg)
		if vetsbp.has_key('Q25434'):
			del vetsbp['Q25434']
		r=self.predict_proba(vetsbp,scale=False)
		scaler=MinMaxScaler() #QuantileTransformer()
		scaler.fit(np.array(r.values()).reshape(-1, 1))
		a=[]
		pickle.dump(scaler, open('marshalled/scaler_new.m', 'w'))
		return scaler

	def learn_distro(self,bg_file='validationDatasets/background/backgroundDataset.fasta',bins=[12,12,12,10,10],reverse=[False,True,True,True,True],minemiss=0.05,plot_histograms=True): # si e' grezza,ma in bin_range_per_fea devi mettere, per ogni feature il numero di bins e il range di ogni feature. questo perche' alcuni valori non vengono visti mai
		seqs=leggifasta(bg_file)
		s={}
		for i in seqs.keys()[:500]:

			if len(seqs[i])>MAX_SEQ_SIZE or len(seqs[i])<MIN_SEQ_SIZE:
				continue
			s[i]=seqs[i]
		seqs=s

		if FAST and 'seqs_vec_background.m' in os.listdir('marshalled'):
			vets,seqs=load_pickle('marshalled/seqs_vec_background.m')
		else:
			vets=self.build_vector(seqs)
		printable=string.printable

		cont=0
		g=[]
		for i in vets.keys():
			dig=vets[i]
			for j in dig:
				l=[]
				for k in j:
					l+=[printable.index(k)]
				g+=[l]
		diz_prob=[]
		diz_cost=[]
		g=np.array(g)
		h=[]
		for i in range(g.shape[1]):
			hist, bin_edges=np.histogram(g[:,i], bins=bins[i], range=(0,bins[i]), density=True,normed=True)
			h+=[hist]
			costante=list(np.cumsum(hist))

			for k in range(len(costante)):

				costante[k]=max(costante[k],minemiss)
				#print costante[i],max(costante[i],minemiss)
				#raw_input()
			#print min(costante),'qe'
			assert min(costante)>=minemiss
			d=[]
			if reverse[i]:
				prev=1.0
				pc=1.0
				c=[1]
				for k in range(1,len(hist[1:])+1):
					c+=[max(c[-1]-hist[k-1],minemiss)]
					assert min(c)>=0.05
				for k in range(len(hist)-1,-1,-1):
					prev-=hist[k]
					prev=max(prev,minemiss)
					d=[prev]+d
				diz_cost+=[c]
				diz_prob+=[d]
			else:
				prev=1.0
				diz_cost+=[costante]

				for k in range(len(hist)):
					prev-=hist[k]
					prev=max(prev,minemiss)
					d+=[prev]
				diz_prob+=[d]

		distro=[]
		for i in diz_prob:
			diz={}
			for k in range(len(i)):
				diz[printable[k]]=i[k]
			distro+=[diz]

		distro_cost=[]
		cont=0
		for i in diz_cost:

			di={}
			for k in range(len(i)):
				#print i[k]
				assert i[k]>=0.05
				di[printable[k]]=i[k]
			distro_cost+=[di]
			cont+=1
		if plot_histograms:
			fea_names=['Sequence complexity','Arg enrichment','Tyr enrichment','RNA-binding','Disorder']
			for i in range(g.shape[1]):
				data = g[:,i]
				fig, ax1 = plt.subplots()
				d = np.diff(np.unique(data)).min()
				left_of_first_bin = data.min() - float(d)/2
				right_of_last_bin = data.max() + float(d)/2
				ax1.hist(data, np.arange(left_of_first_bin, right_of_last_bin + d, d),normed=True,label='feature distribution')

				#ax1.hist(scores, bins=15,alpha=0.3, color="b")
				#ax1.set_xlabel('HMM scores')
				# Make the y-axis label, ticks and tick labels match the line color.
				#ax1.set_ylabel('Occurrence', color='b')
				ax1.tick_params('y', colors='b')

				ax2 = ax1.twinx()
				ax2.tick_params('y', colors='r')
				#values, base = np.histogram(scores, bins=100,normed=False)

				y=[]
				cont=0.0
				x=[]
				for j in sorted(distro_cost[i].keys()):
					if distro_cost[i][j]==0.05:
						y+=[0.0]
					elif distro_cost[i][j]==0.95:
						y+=[1]
					else:
						y+=[distro_cost[i][j]]
					x+=[cont]
					cont+=1

				if i!=0:
					yc=[]
					for k in y:
						yc+=[1-k]
						y=yc

				ax2.plot(x, y, c='r',label='Cumulative')
				if i==1:
					plt.legend()

				#plt.plot(x,y,label='feature cumulative distribution', alpha=0.4)
				plt.legend()

				plt.ylabel('Probability')
				plt.xlabel('Feature value')

				plt.title(fea_names[i])
				plt.savefig(fea_names[i]+'.png',dpi=400)
				plt.clf()
		for i in range(len(distro_cost)):
			v=distro_cost[i].values()
			for k in distro_cost[i].keys():
				distro_cost[i][k]=distro_cost[i][k]/sum(v)
		for i in range(len(distro)):
			v=distro[i].values()
			for k in distro[i].keys():
				distro[i][k]=distro[i][k]/sum(v)
		pickle.dump((distro,distro_cost), open('marshalled/emissions.m', 'w'))
		pickle.dump((distro,distro_cost), open('marshalled/emissions.m', 'w'))
		return distro,distro_cost

class scoring_function:
	def __init__(self):
		self.distro,self.distro_cost=load_pickle('marshalled/emissions.m')
		seqs_rna=leggifasta('rna_bp.fasta')
		s={}
		for i in seqs_rna.keys()[:]:
			s[i]=seqs_rna[i]
		seqs_rna=s
		seqs=leggifasta('phase_separation_proteins.fasta')

		s={}
		for i in seqs.keys()[:]:
			s[i]=seqs[i]
		seqs=s
		self.vets=build_vector(seqs)
		self.vets_rna=build_vector(seqs_rna)

	def score(self,c):
		model=phase_hmm(self.distro,self.distro_cost,param=c,optim_params=True)
		a= model.predict(self.vets)
		b= model.predict(self.vets_rna)
		return np.median(b)-np.median(a)

def parameter_opt():
	score_obj=scoring_function()
	c=minimize(score_obj.score, [0,70,80,50,30])
	print (c)

def discretizza(feas,continue_val=[False,False,False,False,True],n_bins=10): ## metti numero bins per ogni feature
	printable=string.printable
	bins = numpy.linspace(0, 1, n_bins)
	dig=[]
	for i in range(len(continue_val)):
		if continue_val[i]:
			dig+=[numpy.digitize(feas[i], bins)]
			#print numpy.digitize(feas[i], bins)

		else:

			dig+=[feas[i]]
	v=[]
	for i in range(len(dig[0])):
		s=''
		for j in range(len(dig)):
			#print len(feas[j]),len(feas),len(threshold),i,j
			s+=printable[dig[j][i]]
		v+=[s]
	return v

####  cose da fare : cumulativa background direttamente da hist, le distro fatte  bene--> diverso[i], costante[i] per fare background o non background
PSP = 'validationDatasets/phaseSepProts/phase_separation_proteins.fasta'
FAST = True
#TARGET = "validationDatasets/mip6.fasta"
#TARGET = "proteomes/ecoli.UP000000625.fasta"
#TARGET = "proteomes/saccharomycesUP000002311.fasta"
#TARGET= "validationDatasets/144granuleCoreProts/144granuleCoreProts.fasta"
#TARGET = "validationDatasets/granuleForming/granuleForming.fasta"
#TARGET = "validationDatasets/RNAbp/RNABPdatasetFinal.fasta"
#TARGET = "disordered.fasta"
#TARGET = "validationDatasets/prionLikeFromCellPaper/prionLike.fasta"
#TARGET = "validationDatasets/prions/prionDataset.fasta"
TARGET = "validationDatasets/background/backgroundDataset.fasta"
#TARGET= "validationDatasets/disordered_proteins.fasta" #
#TARGET='newPSP.fasta'
ERASE = False
MIN_SEQ_SIZE=150
MAX_SEQ_SIZE=3000

def test(TARGET):
	model=phase_hmm()
	model.fit()


	seqs = leggifasta(PSP)
	targetSeqs = leggifasta(TARGET)
	print ("Found %d proteins in %s" % (len(seqs), PSP))
	print ("Reading PSP...")
	if FAST and os.path.exists("marshalled/phaseSepVects.cPickle") and not ERASE:
		vets = load_pickle("marshalled/phaseSepVects.cPickle")
	else:
		s={}
		for i in seqs.keys()[:]:
			if len(seqs[i])>MAX_SEQ_SIZE :
				continue
			s[i]=seqs[i]
		seqs=s
		vets = model.build_vector(seqs)
		pickle.dump(vets, open("marshalled/phaseSepVects.cPickle","w"))
	phaseSepPreds = model.predict_proba(vets).values()
	####################################################################

	if  "background" in TARGET:
		targets = targetSeqs.keys()[15000:]

		random.shuffle(targets)
		targets = targets[:2000]

	else:
		targets = targetSeqs.keys()[:]

	print ("Found %d proteins in %s" % (len(targets), TARGET))

	if FAST and os.path.exists("marshalled/"+TARGET.split("/")[-1]+".cPickle") and not ERASE:
		vets = load_pickle("marshalled/"+TARGET.split("/")[-1]+".cPickle")


	else:
		s={}
		for i in targets:
			if len(targetSeqs[i])>MAX_SEQ_SIZE or len(targetSeqs[i])<MIN_SEQ_SIZE:
				continue
			s[i]=targetSeqs[i]
		targets=s

		targets = removeKnownPSP(targets, seqs.keys())  #this removes known PSP

		vets = model.build_vector(targets)
		pickle.dump(vets, open("marshalled/"+TARGET.split("/")[-1]+".cPickle","w"))

	print ('@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@')
	preds = model.predict_proba(vets)
	prediction_diz=preds
	for i in sorted(preds.items(), key=lambda x:x[1], reverse=True)[0:50]:
		print (i[0],i[1],'\\\\')

	targetPreds = preds.values()
	print (targetPreds)
	print ("Using %d proteins" % len(vets))

	####random.shuffle(preds)
	labels = [1]*len(phaseSepPreds)+[0]*len(targetPreds)
	print ('auc',roc_auc_score(labels, preds))
	print ('pvalue',ranksums(phaseSepPreds, targetPreds))
	print (np.median(phaseSepPreds))

	plt.figure(figsize=(5,4))
	plt.title("Ranksums p-value = %.3e\nAUC = %.3f" % (ranksums(phaseSepPreds,targetPreds)[1], roc_auc_score(labels, preds)))
	plt.grid()
	plt.ylabel("HMM scores")
	#plt.violinplot([a,b])
	plt.boxplot([phaseSepPreds, targetPreds], notch=True)

	plt.xticks([1,2],["Phase Separation", "IDPs"])
	plt.tight_layout()
	#plt.savefig(TARGET+".png", dpi=400)
	#plt.show()

	print ('ok')

	return prediction_diz

def removeKnownPSP(t, s):
	#print type(t), type(s)
	r = {}
	c = 0
	for i in t.items():
		if i[0] in s:
			c += 1
			continue
		r[i[0]] = i[1]
	print ("Removed %d prots from targets" % c)
	return r

if __name__ == '__main__':
	test(TARGET)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  learnBackgrounds.py
#
#  Copyright 2018 Daniele Raimondi <daniele.raimondi@vub.be>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
import math, os, gc
import matplotlib.pyplot as plt
import numpy as np


def readFASTA(seqFile, MIN_LENGTH = 20, MAX_LENGTH=1500):
	ifp = open(seqFile, "r")
	sl = {}
	i = 0
	line = ifp.readline()
	discarded = 0

	while len(line) != 0:
		tmp = []
		if line[0] == '>':
			#sl.append([line.strip().replace("|","").replace(">","")[:5]+"_",""])	#only for spx

			tmp = [line.strip().split(" ")[-3][3:],""] #attenzione
			line = ifp.readline()
			while len(line) > 0 and line[0] != '>':
				tmp[1] = tmp[1] + line.strip()
				line = ifp.readline()
			#print len(tmp[1])
			#raw_input()
			i = i + 1
			if len(tmp[1]) > MAX_LENGTH or len(tmp[1]) < MIN_LENGTH:
				#print "discard"
				discarded += 1
				continue
			else:
				sl[tmp[0]] = tmp[1]
		else:
			raise Exception("Syntax error in the fasta file "+seqFile+"!")
	print ("Found %d sequences, added %d discarded %d" % (i, len(sl), discarded))
	return sl

listAA = [  "A",  "C",  "D",  "E",  "F",  "G",  "H",  "I",  "K",  "L",  "M",  "N",  "P",  "Q",  "R",  "S",  "T",  "V",  "W",  "Y"  ]

def buildVectorsRSA(seqs, WS):
	X = []
	for u in seqs.items():
		i = 0
		#assert len(tmprsa) == len(seq)
		targetSeq = u[1]
		#print targetSeq
		while i < len(str(targetSeq)):
			fragment = seq2vecSklearn(targetSeq, WS, i)
			X.append(fragment)
			#print fragment
			#raw_input()
			i += 1
	#raw_input()
	return X

def seq2vecSklearn(seq, sw, pos):
	vec=[]
	startpos=pos-(sw/2)
	if startpos < 0 :
		vec= ["0"] * int((-startpos))
		startpos=0
	endpos=pos+(sw/2)+1

	vec += str(seq)[int(startpos):int(min(len(str(seq)),endpos))]
	if endpos > len(str(seq)):
		endpos = endpos - len(str(seq))
		vec += ["0"]* int(endpos)
	#assert len(vec) == sw
	return vec

def buildSmoothVect(seq, WS):
	targetSeq = seq
	X = []
	#print targetSeq
	i = 0
	while i < len(targetSeq):
		fragment = smoothVect1(targetSeq, WS, i)
		X.append(np.mean(fragment))
		i += 1
	#raw_input()
	return X

def smoothVect1(seq,sw, pos):
	vec=[]
	startpos=pos-(sw/2)
	if startpos < 0 :
		vec= [0] * (-startpos)
		startpos=0
	endpos=pos+(sw/2)+1

	vec += seq[startpos:min(len(seq),endpos)]
	if endpos > len(seq):
		endpos = endpos - len(seq)
		vec += [0]* endpos
	#assert len(vec) == sw
	return vec

def leggifasta(database):
		f=open(database)
		uniprot=f.readlines()
		f.close()
		dizio={}
		for i in uniprot:
			i=i.strip()
			if '>' in i:
				if "|" in i:
					uniprotid=i.strip('>\n').split('|')[1]
				else:
					uniprotid = i.strip(">\n")
				dizio[uniprotid]=''
			else:
				dizio[uniprotid]=dizio[uniprotid]+i.strip('\n').upper()
		return dizio

def seqs_features(seqs, WS=10):
	diz={}
	for id in seqs.keys():
		seq=seqs[id]
		lcomp = []
		r = []
		g = []
		y = []
		fragments = buildVectorsRSA({"pippo":seq}, WS)
		for c, f in enumerate(fragments):
			c = len(set(fragments[c]))
			lcomp.append(c)
			#r.append(f.count("R"))
			r.append(f.count("R"))
			y.append(f.count("Y"))
		diz[id]=( lcomp, r, y)
	return diz

def main():
	print (seqs_features({'ddd':'AAAAAAAAAAAAAAAAAAACCCCCCCCCCCCCCTTTTTTTTTTTTTRRRRRRRRRRRRRRRRRRRGGGGGGGGGGGGGGGGG'}))

def getLKcomp(distro, frag):
	c = len(set(frag))
	return distro[c]

def getLKfreqMultivar(distro, frag):
	h = getHash(frag)
	r = 0
	try:
		return distro[h]
	except:
		return np.nan

def getHash(f):
	l = []
	for i in listAA:
		l.append(f.count(i))
	return hash(str(l))

def getLKfreq(distro, frag):
	r = 1
	for aa in frag:
		if aa not in listAA:
			continue
		r *= distro[aa]
	return r

if __name__ == '__main__':
	print (main())



"""
======================COPYRIGHT/LICENSE START==========================

projectIO.py: I/O for nmrStar project file

Copyright (C) 2005-2008 Wim Vranken (European Bioinformatics Institute)

=======================================================================

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

A copy of this license can be found in ../../../../license/LGPL.license

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)
- PDBe website (http://www.ebi.ac.uk/pdbe/)

- contact Wim Vranken (wim@ebi.ac.uk)
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
"""

from b2bTools.general.ccpn.format.nmrStar.sequenceIO import NmrStarSequenceFile
from b2bTools.general.ccpn.format.nmrStar.chemShiftsIO import NmrStarChemShiftFile
from b2bTools.general.ccpn.format.nmrStar.orderParamIO import NmrStarOrderParamFile
from b2bTools.general.ccpn.format.nmrStar.hExchRateIO import NmrStarHExchRateFile
from b2bTools.general.ccpn.format.nmrStar.hExchProtectionIO import NmrStarHExchProtectionFile
from b2bTools.general.ccpn.format.nmrStar.t1RelaxIO import NmrStarT1RelaxFile
from b2bTools.general.ccpn.format.nmrStar.t1RhoRelaxIO import NmrStarT1RhoRelaxFile
from b2bTools.general.ccpn.format.nmrStar.t2RelaxIO import NmrStarT2RelaxFile
from b2bTools.general.ccpn.format.nmrStar.hetNoeIO import NmrStarHetNoeFile
from b2bTools.general.ccpn.format.nmrStar.jCouplingIO import NmrStarJCouplingFile
from b2bTools.general.ccpn.format.nmrStar.rdcIO import NmrStarRdcFile
from b2bTools.general.ccpn.format.nmrStar.distanceConstraintsIO import NmrStarDistanceConstraintFile
from b2bTools.general.ccpn.format.nmrStar.dihedralConstraintsIO import NmrStarDihedralConstraintFile
from b2bTools.general.ccpn.format.nmrStar.rdcConstraintsIO import NmrStarRdcConstraintFile
from b2bTools.general.ccpn.format.nmrStar.coordinatesIO import NmrStarCoordinateFile
from b2bTools.general.ccpn.format.nmrStar.peopleAndCitationsIO import NmrStarPeopleAndCitationsFile

from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarFile

from b2bTools.general.ccpn.universal.Util import returnFloat

#####################
# Class definitions #
#####################

class NmrStarProjectFile(NmrStarFile):

  # Information on file level

  def initialize(self, version = '2.1.1'):

    # These are fully FC supported (in DataFormat.py)
    self.distanceConstraintFiles = []
    self.hBondConstraintFiles = []
    self.dihedralConstraintFiles = []
    self.rdcConstraintFiles = []
    self.chemShiftFiles = []
    self.orderParamFiles = []
    self.hExchRateFiles = []
    self.hExchProtectionFiles = []
    self.t1RelaxFiles = []
    self.t1RhoRelaxFiles = []
    self.t2RelaxFiles = []
    self.hetNoeFiles = []
    self.jCouplingFiles = []
    self.rdcFiles = []
    self.sequenceFiles = []
    self.coordinateFiles = []
    self.peopleAndCitationsFiles = []

    # These are only available for 3.1 and on NMR-STAR project level reading
    self.entryInformation = None
    self.studyFiles = []
    self.sampleFiles = []
    self.sampleConditionFiles = []
    self.softwareFiles = []
    self.methodFiles = []
    self.confStatFiles = []
    self.instrumentFiles = []
    self.probeFiles = []
    self.nmrExperimentFiles = []
    self.chemShiftRefFiles = []
    self.sourceFiles = []

    if not self.version:
      self.version = version

    self.components = ['all']

    self.setComponents()

  def read(self,verbose = 0,**keywds):

    #
    # Read is now version 2.1.1 and 3.0
    #

    if not self.checkVersion():
      return False

    self.readGeneric(verbose = verbose)

    if not self.readStatusCheck():
      return False

    MRFileBlockName = '_BMRB_dev_Position_block_in_MR_file'
    MRFileSaveFrame = 'MR_file_characteristics'
    cyanaLibUsed = 0

    #
    # Distance and H-bond constraints
    #

    saveFrameNames = ['distance_constraints' ,'general_distance_constraints']

    for saveFrameName in saveFrameNames:
      if saveFrameName in self.sfs:

        for saveFrame in self.sfs[saveFrameName]:

          isHBond = 0

          #
          # Check whether it's an H bond constraint bit
          #

          if self.version == '2.1.1':

            if MRFileBlockName in saveFrame.tags.keys():
              if MRFileSaveFrame in self.sfs:
                loop = self.sfs[MRFileSaveFrame][0].tables[MRFileBlockName]
                loopIndex = loop.tags[MRFileBlockName].index(saveFrame.tags[MRFileBlockName])

                if loop.tags['_BMRB_dev_Subtype'][loopIndex] == "hydrogen bond":
                  isHBond = 1

          elif self.version == '3.0':

            if 'Subtype' in saveFrame.tags and saveFrame.tags['Subtype'] == 'hydrogen bond':
              isHBond = 1

          elif self.version == '3.1':

            if 'Constraint_type' in saveFrame.tags and saveFrame.tags['Constraint_type'] == 'hydrogen bond':
              isHBond = 1

          constraintFile = NmrStarDistanceConstraintFile(self.name,self,saveFrame)

          if not isHBond:
            self.distanceConstraintFiles.append(constraintFile)
          else:
            self.hBondConstraintFiles.append(constraintFile)

    #
    # Dihedral constraints
    #

    saveFrameName = 'torsion_angle_constraints'

    if saveFrameName in self.sfs:

      for saveFrame in self.sfs[saveFrameName]:

        constraintFile = NmrStarDihedralConstraintFile(self.name,self,saveFrame)
        self.dihedralConstraintFiles.append(constraintFile)

        if constraintFile.cyanaLibUsed:
          cyanaLibUsed = 1


    #
    # RDC constraints
    #

    if self.version == '3.1':
      saveFrameName = 'RDC_constraints'
    #else:
    #  saveFrameName = 'residual_dipolar_couplings'

    if saveFrameName in self.sfs:

      for saveFrame in self.sfs[saveFrameName]:

        constraintFile = NmrStarRdcConstraintFile(self.name,self,saveFrame)
        self.rdcConstraintFiles.append(constraintFile)

        if constraintFile.cyanaLibUsed:
          cyanaLibUsed = 1

    if cyanaLibUsed:
      print ("Using CYANA library - courtesy of Peter Guentert.")

    #
    # Sequence
    #

    saveFrameName = None
    if self.version == '2.1.1':
      saveFrameName = 'monomeric_polymer' # Using this because assembly saveframe not always present...
    elif self.version[0] == '3':
      saveFrameName = 'assembly'

    if saveFrameName in self.sfs:

      for saveFrame in self.sfs[saveFrameName]:
        self.sequenceFiles.append(NmrStarSequenceFile(self.name,self,saveFrame))

    #
    # Coordinates
    #

    saveFrameName = None
    if self.version == '2.1.1':
      saveFrameName = 'conformer_family_coord_set' # TODO does this exist?
    elif self.version[0] == '3':
      saveFrameName = 'conformer_family_coord_set'

    if saveFrameName in self.sfs:

      localKeywds = {}
      if 'maxNum' in keywds:
        localKeywds['maxModelNum'] = keywds['maxNum']
      for saveFrame in self.sfs[saveFrameName]:
        self.coordinateFiles.append(NmrStarCoordinateFile(self.name,self,saveFrame,**localKeywds))

    #
    # Chemical shifts
    #

    saveFrameName = 'assigned_chemical_shifts'

    if saveFrameName in self.sfs:

      for saveFrame in self.sfs[saveFrameName]:

        chemShiftFile = NmrStarChemShiftFile(self.name,self,saveFrame)

        tagNameDict = {'Details':            'details',
                       'Chem_shift_1H_err':  'chemShiftErr1H',
                       'Chem_shift_13C_err': 'chemShiftErr13C',
                       'Chem_shift_15N_err': 'chemShiftErr15N',
                       'Chem_shift_31P_err': 'chemShiftErr31P',
                       'Chem_shift_2H_err':  'chemShiftErr2H',
                       'Chem_shift_19F_err': 'chemShiftErr19F'}

        for tagName in tagNameDict.keys():
          if tagName in saveFrame.tags:
            setattr(chemShiftFile, tagNameDict[tagName], saveFrame.tags[tagName])
          else:
            setattr(chemShiftFile, tagNameDict[tagName], None)

        self.chemShiftFiles.append(chemShiftFile)

    #
    # J couplings
    #

    saveFrameName = 'coupling_constants'

    if saveFrameName in self.sfs:

      for saveFrame in self.sfs[saveFrameName]:

        jCouplingFile = NmrStarJCouplingFile(self.name,self,saveFrame)
        self.jCouplingFiles.append(jCouplingFile)

    #
    # RDCs
    #

    saveFrameName = None
    if self.version == '2.1.1':
      saveFrameName = 'residual_dipolar_couplings'
    elif self.version[0] == '3':
      saveFrameName = 'RDCs'

    if saveFrameName in self.sfs:

      for saveFrame in self.sfs[saveFrameName]:

        rdcFile = NmrStarRdcFile(self.name,self,saveFrame)
        self.rdcFiles.append(rdcFile)

    #
    # Order parameters
    #

    saveFrameName = None
    if self.version == '2.1.1':
      saveFrameName = 'S2_parameters'
    elif self.version[0] == '3':
      saveFrameName = 'order_parameters'

    if saveFrameName in self.sfs:

      for saveFrame in self.sfs[saveFrameName]:

        orderParamFile = NmrStarOrderParamFile(self.name,self,saveFrame)
        self.orderParamFiles.append(orderParamFile)

    #
    # People and citations
    #

    if self.version[0] == '3':
      for saveFrameName in ('entry_information','citations'):
        if saveFrameName in self.sfs:
          if saveFrameName == 'entry_information':
            saveFrame = self.sfs[saveFrameName][0] # Only ever one...
            self.peopleAndCitationsFiles.append(NmrStarPeopleAndCitationsFile(self.name,self,saveFrame,'contactPersons'))
            self.peopleAndCitationsFiles.append(NmrStarPeopleAndCitationsFile(self.name,self,saveFrame,'authors'))
          else:
            for saveFrame in self.sfs[saveFrameName]:
              self.peopleAndCitationsFiles.append(NmrStarPeopleAndCitationsFile(self.name,self,saveFrame,None))

    #
    # Project level information, only in 3.1 (and higher, eventually)
    #

    if self.version == '3.1':

      # This is more or less in order of importance!

      # Additional entry level information
      saveFrameName = 'entry_information'
      if saveFrameName in self.sfs:
        # Strictly speaking this is not necessary, but it is for hacking the correct NMR-STAR version
        # from two entry_information saveframes (which is illegal) in the NRG/remediated wwPDB project
        for saveFrame in self.sfs[saveFrameName]:
          self.entryInformation = NmrStarEntryInformation(self.name,self,saveFrame)

      # Studies
      saveFrameName = 'study_list'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.studyFiles.append(NmrStarStudyFile(self.name,self,saveFrame))

      # Sample conditions
      saveFrameName = 'sample_conditions'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.sampleConditionFiles.append(NmrStarSampleConditionsFile(self.name,self,saveFrame))

      # Chemical shift referencing
      saveFrameName = 'chem_shift_reference'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.chemShiftRefFiles.append(NmrStarChemShiftRefFile(self.name,self,saveFrame))

      # Sample information
      saveFrameName = 'sample'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.sampleFiles.append(NmrStarSampleFile(self.name,self,saveFrame))

      # NMR experiments
      saveFrameName = 'experiment_list'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.nmrExperimentFiles.append(NmrStarExperimentFile(self.name,self,saveFrame))

      # Instruments
      saveFrameName = 'NMR_spectrometer_list'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.instrumentFiles.append(NmrStarInstrumentFile(self.name,self,saveFrame))

      # Probes
      saveFrameName = 'NMR_spectrometer_probe'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.probeFiles.append(NmrStarProbeFile(self.name,self,saveFrame))

      # Software
      saveFrameName = 'software'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.softwareFiles.append(NmrStarSoftwareFile(self.name,self,saveFrame))

      # Methods
      saveFrameName = 'method'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.methodFiles.append(NmrStarMethodFile(self.name,self,saveFrame))

      # Ensemble stats
      saveFrameName = 'conformer_statistics'
      if saveFrameName in self.sfs:
        for saveFrame in self.sfs[saveFrameName]:
          self.confStatFiles.append(NmrStarConfStatFile(self.name,self,saveFrame))

      # Natural/experimental source
      for saveFrameName in ('natural_source','experimental_source'):
        if saveFrameName in self.sfs:
          for saveFrame in self.sfs[saveFrameName]:
            self.sourceFiles.append(NmrStarSourceFile(self.name,self,saveFrame,saveFrameName))

      # H-Exchange Rates
      saveFrameName = 'H_exch_rates'

      if saveFrameName in self.sfs:

        for saveFrame in self.sfs[saveFrameName]:

          hExchRateFile = NmrStarHExchRateFile(self.name,self,saveFrame)
          self.hExchRateFiles.append(hExchRateFile)

      # H-Protection Values
      saveFrameName = 'H_exch_protection_factors'

      if saveFrameName in self.sfs:

        for saveFrame in self.sfs[saveFrameName]:

          hExchProtectionFile = NmrStarHExchProtectionFile(self.name,self,saveFrame)
          self.hExchProtectionFiles.append(hExchProtectionFile)

      # T1 relaxation
      saveFrameName = 'heteronucl_T1_relaxation'

      if saveFrameName in self.sfs:

        for saveFrame in self.sfs[saveFrameName]:

          t1RelaxFile = NmrStarT1RelaxFile(self.name,self,saveFrame)
          self.t1RelaxFiles.append(t1RelaxFile)

      # T1Rho relaxation
      saveFrameName = 'heteronucl_T1rho_relaxation'

      if saveFrameName in self.sfs:

        for saveFrame in self.sfs[saveFrameName]:

          t1RhoRelaxFile = NmrStarT1RhoRelaxFile(self.name,self,saveFrame)
          self.t1RhoRelaxFiles.append(t1RhoRelaxFile)

      # T2 relaxation
      saveFrameName = 'heteronucl_T2_relaxation'

      if saveFrameName in self.sfs:

        for saveFrame in self.sfs[saveFrameName]:

          t2RelaxFile = NmrStarT2RelaxFile(self.name,self,saveFrame)
          self.t2RelaxFiles.append(t2RelaxFile)

      # NOEs
      saveFrameName = 'heteronucl_NOEs'

      if saveFrameName in self.sfs:

        for saveFrame in self.sfs[saveFrameName]:

          hetNoeFile = NmrStarHetNoeFile(self.name,self,saveFrame)
          self.hetNoeFiles.append(hetNoeFile)

    return True

  def readStatusCheck(self):

    #
    # Dummy function that can be used elsewhere to customize the nmrStar project
    # reading
    #

    return True

  def write(self,verbose = 0):

    #
    # This is taken over by NmrStarExport code for 3.1 and up. Only useful for 2.1.1
    #

    self.writeGeneric(verbose = verbose)

#
# Classes specific for NMR-STAR project level reading, some might be split out to IO.py files if become
# part of generic DataFormat data component handling.
#
# Note that the way this is set up is 'overkill', but necessary in case we need to fit this in into
# the DataFormat.py structure.
#

from b2bTools.general.ccpn.format.nmrStar.generalIO import NmrStarGenericFile

class NmrStarProjectDataComponent(NmrStarGenericFile):

  """
  Information on file level.
  """

  def genericInit(self,parent,saveFrame):

    self.saveFrame = saveFrame

    self.parent = parent
    self.version = parent.version

    if self.saveFrame:
      self.parseSaveFrame()

  def checkVersion(self):

    if self.version not in ('3.1',):
      print ("  Cannot parse nmrStar distance constraints for version %s" % self.version)
      return False

    return True

#
# Entry level information
#

class NmrStarEntryInformation(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.entry = None
    self.entryId = None
    self.entrySrcs = []
    self.relatedEntries = []
    self.structKeywds = []
    self.releaseInfo = []
    self.strucGenomInfo = []

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.entryId = self.saveFrame.tags['ID']

    infoList = []

    tagNames = ['Title', 'Details', 'Experimental_method', 'Experimental_method_subtype',
                'Special_processing_instructions', 'Version_type', 'Submission_date',
                'Accession_date', 'Origination', 'NMR_STAR_version', 'Original_NMR_STAR_version',
                'Dep_release_code_coordinates', 'Dep_release_code_nmr_constraints',
                'Dep_release_code_nmr_exptl', 'Dep_release_code_sequence',
                'CASP_target', 'Update_BMRB_accession_code', 'Replace_BMRB_accession_code',
                'Update_PDB_accession_code', 'Replace_PDB_accession_code',
                'BMRB_update_details', 'PDB_update_details', 'Release_request',
                'PDB_deposit_site', 'PDB_process_site', 'BMRB_deposit_site',
                'BMRB_process_site', 'RCSB_annotator', 'Assigned_BMRB_ID',
                'Assigned_BMRB_deposition_code', 'Assigned_PDB_ID', 'Assigned_PDB_deposition_code',
                'Date_nmr_constraints', 'Recvd_author_approval', 'PDB_date_submitted',
                'Author_release_status_code', 'Author_approval_type']

    for tagName in tagNames:
      # Possible that tag is not there (if it was empty)... take this into account
      if tagName in self.saveFrame.tags:
        value = self.saveFrame.tags[tagName]
      else:
        value = None

      infoList.append(value)

    #
    # Note: this is a NASTY HACK to get correct NMR_STAR_VERSION out of 'joint' coordinate/restraint file in NRG
    # and remediated wwPDB project. Should not be relevant for any other file.
    #

    if hasattr(self.parent,'entryInformation') and self.parent.entryInformation:
      if self.parent.entryInformation.entry.nmrStarVers:
        infoList[tagNames.index('NMR_STAR_version')] = self.parent.entryInformation.entry.nmrStarVers

    #
    # END nasty hack
    #

    self.entry = Entry(self,*infoList)

    self.tableName1 = '_Entry_src'
    tagNames = ('Organization_full_name',)

    if self.tableName1 in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName1].tags

      for i in range(0,len(tableTags['Organization_full_name'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.entrySrcs.append(EntrySrc(self,*infoList))

    self.tableName2 = '_Related_entries'
    tagNames = ('Database_name', 'Database_accession_code', 'Relationship')

    if self.tableName2 in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName2].tags

      for i in range(0,len(tableTags['Database_name'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.relatedEntries.append(RelatedEntry(self,*infoList))

    self.tableName3 = '_Struct_keywords'
    tagNames = ('Entry_ID','Keywords','Text')

    if self.tableName3 in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName3].tags

      for i in range(0,len(tableTags['Entry_ID'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.structKeywds.append(StructKeyWord(self,*infoList))

    self.tableName4 = '_Release'
    tagNames = ('Entry_ID','Release_number','Date','Submission_date','Type','Author','Detail')

    if self.tableName4 in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName4].tags

      for i in range(0,len(tableTags['Entry_ID'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.releaseInfo.append(Release(self,*infoList))

    self.tableName5 = '_SG_project'
    tagNames = ('Project_name','Full_name_of_center')

    if self.tableName5 in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName5].tags

      for i in range(0,len(tableTags['Project_name'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.strucGenomInfo.append(StrucGenomics(self,*infoList))

class Entry:

  def __init__(self,parent,title,details,exptMethod,exptMethodSub,specProcInstr,versionType,subDate,accDate,
               origin,nmrStarVers,origNmrStarVers,depRelCoord,depRelConstr,depRelNmr,depRelSeq,
               caspTarget,updBmrbCode,replBmrbCode,updPdbCode,replPdbCode,bmrbUpdDet,pdbUpdDet,relReq,
               pdbDepSite,pdbProcSite,bmrbDepSite,bmrbProcSite,rcsbAnn,assBmrbId,assBmrbCode,assPdbId,assPdbCode,
               dateNmrConstr, recAuthAppr, pdbDateRec, authRelStatCode, authApprType):

    self.parent = parent

    self.title = title
    self.details = details

    self.exptMethod = exptMethod
    self.exptMethodSub = exptMethodSub
    self.specProcInstr = specProcInstr

    self.versionType = versionType
    self.subDate = subDate
    self.accDate = accDate
    self.origin = origin
    self.nmrStarVers = nmrStarVers
    self.origNmrStarVers = origNmrStarVers
    self.depRelCoord = depRelCoord
    self.depRelConstr = depRelConstr
    self.depRelNmr = depRelNmr
    self.depRelSeq = depRelSeq
    self.caspTarget = caspTarget
    self.updBmrbCode = updBmrbCode
    self.replBmrbCode = replBmrbCode
    self.updPdbCode = updPdbCode
    self.replPdbCode = replPdbCode
    self.bmrbUpdDet = bmrbUpdDet
    self.pdbUpdDet = pdbUpdDet
    self.relReq = relReq
    self.pdbDepSite = pdbDepSite
    self.pdbProcSite = pdbProcSite
    self.bmrbDepSite = bmrbDepSite
    self.bmrbProcSite = bmrbProcSite
    self.rcsbAnn = rcsbAnn
    self.assBmrbId = assBmrbId
    self.assBmrbCode = assBmrbCode
    self.assPdbId = assPdbId
    self.assPdbCode = assPdbCode
    self.dateNmrConstr = dateNmrConstr
    self.recAuthAppr = recAuthAppr
    self.pdbDateRec = pdbDateRec
    self.authRelStatCode = authRelStatCode
    self.authApprType = authApprType

class EntrySrc:

  def __init__(self,parent,orgFullName):

    self.parent = parent
    self.orgFullName = orgFullName

class RelatedEntry:

  def __init__(self,parent,dbName,dbAcc,reln):

    self.parent = parent
    self.dbName = dbName
    self.dbAcc = dbAcc
    self.reln = reln

class StructKeyWord:

  def __init__(self,parent,entryId,keywd,text):

    self.parent = parent
    self.entryId = entryId
    self.keywd = keywd
    self.text = text

class Release:

  def __init__(self,parent,entryId,relNo,date,subDateRel,relType,author,detail):

    self.parent = parent
    self.entryId = entryId
    self.relNo = relNo
    self.date = date
    self.subDateRel = subDateRel
    self.relType = relType
    self.author = author
    self.detail = detail

class StrucGenomics:

  def __init__(self,parent,projName,centerFullName):

    self.parent = parent
    self.projName = projName
    self.centerFullName = centerFullName

#
# Studies information
#

class NmrStarStudyFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.studyLists = []
    self.studies = []
    self.studyKeywds = []
    self.studyId = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.studyId = self.saveFrame.tags['ID']

    infoList = []

    infoList.append(self.saveFrame.title) # Safer to use title here - sf_framecode may be missing (Wim)

    self.studyLists.append(StudyList(self,*infoList))

    self.tableName1 = '_Study'
    tagNames = ('ID','Name','Type','Details')

    if self.tableName1 in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName1].tags

      for i in range(0,len(tableTags['ID'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.studies.append(Study(self,*infoList))

    self.tableName2 = '_Study_keyword'
    tagNames = ('Study_ID','Keyword')

    if self.tableName2 in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName2].tags

      for i in range(0,len(tableTags['Study_ID'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.studyKeywds.append(StudyKeyWord(self,*infoList))


class StudyList:

  def __init__(self,parent,name):

    # Warning: need to do some retyping here, data type not always correct in original NMR-STAR

    self.parent = parent
    self.name = name

class Study:

  def __init__(self,parent,Id,name,type,details):

    self.parent = parent
    self.Id = Id
    self.name = name
    self.type = type
    self.details = details

class StudyKeyWord:

  def __init__(self,parent,studyId,keywd):

    self.parent = parent
    self.studyId = studyId
    self.keywd = keywd

#
# Sample conditions information
#

class NmrStarSampleConditionsFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    # These two are specific for the type of information we're reading
    self.sampleConditions = []
    self.sampleConditionSetId      = None
    self.sampleConditionSetName    = None
    self.sampleConditionSetDetails = None

    self.genericInit(parent,saveFrame)
    self.sampleConditionSetId = None

    self.genericInit(parent,saveFrame) # This always has to be here

  def parseSaveFrame(self):

    if not self.checkVersion():  # This always has to be here
      return

    self.sampleConditionSetId   = self.saveFrame.tags['ID']
    self.sampleConditionSetName = self.saveFrame.title

    if 'Details' in self.saveFrame.tags:
      self.sampleConditionSetDetails = self.saveFrame.tags['Details']

    #if self.sampleConditionSetId in self.sampleConditions:
    #  return

    self.tableName = '_Sample_condition_variable'      # This has to be the table 'prefix' before the . as it appears in the NMR-STAR file
    tagNames = ('Type','Val','Val_err','Val_units')
    if self.tableName in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName].tags

      for i in range(len(tableTags['Val'])):    # Just pick any obligatory tag from the table to loop over the rows

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.sampleConditions.append(SampleCondition(self,*infoList)) # So self is the 'parent' of the SampleCondition data class

# TODO: this might be better kept in ccp.general.formatIO?
class SampleCondition:

  def __init__(self,parent,type,value,error,units):

    # Warning: need to do some retyping here, data type not always correct in original NMR-STAR

    self.parent = parent
    self.type = type

    self.value = None

    # Exceptions
    if type == 'pressure' and value == 'ambient':
      self.value = 1.0
    elif value:
      self.value = returnFloat(value)

    self.error = None

    if error:
      self.error = returnFloat(error)

    self.units = units

#
# Chemical shift referencing information
#

class NmrStarChemShiftRefFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.chemShiftReferences = []
    self.chemShiftRefId = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.chemShiftRefId = self.saveFrame.tags['ID']

    infoList = []

    tagNames = ('Proton_shifts_flag','Carbon_shifts_flag','Nitrogen_shifts_flag','Phosphorus_shifts_flag','Other_shifts_flag')

    for tagName in tagNames:
      if tagName in self.saveFrame.tags:
        value = self.saveFrame.tags[tagName]
      else:
        value = None

      infoList.append(value)

    self.chemShiftRef = ChemShiftRefList(self,*infoList)

    self.tableName1 = '_Chem_shift_ref'
    tagNames = ('Atom_type','Atom_isotope_number','Mol_common_name','Atom_group','Chem_shift_units','Chem_shift_val','Ref_method','Ref_type','Indirect_shift_ratio','External_ref_loc','External_ref_sample_geometry','External_ref_axis')

    if self.tableName1 in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName1].tags

      for i in range(0,len(tableTags['Atom_type'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.chemShiftReferences.append(ChemShiftRef(self,*infoList))

class ChemShiftRefList:

  def __init__(self,parent,protonFlag,carbonFlag,nitrogenFlag,phosphorusFlag,otherFlag):

    self.parent = parent
    self.protonFlag = protonFlag
    self.carbonFlag = carbonFlag
    self.nitrogenFlag = nitrogenFlag
    self.phosphorusFlag = phosphorusFlag
    self.otherFlag = otherFlag

class ChemShiftRef:

  def __init__(self,parent,atomType,isotopeNum,molName,atomGroup,chemShiftUnits,chemShiftVal,refMeth,refType,indirectShiftRatio,location = None,geometry = None, axis = None):

    self.parent = parent
    self.atomType = atomType
    self.isotopeNum = isotopeNum
    self.molName = molName
    self.atomGroup = atomGroup
    self.chemShiftUnits = chemShiftUnits
    self.chemShiftVal = chemShiftVal
    self.refMeth = refMeth
    self.refType = refType
    self.indirectShiftRatio = indirectShiftRatio
    self.location = location
    self.geometry = geometry
    self.axis = axis

#
# Sample information
#

class NmrStarSampleFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.samples = []
    self.sampleComponents = []
    self.sampleId = None
    self.sampleDetails = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.sampleId = self.saveFrame.tags['ID']

    infoList = []

    infoList.append(self.saveFrame.title)

    tagNames = ('Type','Details','Solvent_system','Aggregate_sample_number')

    for tagName in tagNames:
      if tagName in self.saveFrame.tags:
        value = self.saveFrame.tags[tagName]
      else:
        value = None

      infoList.append(value)

    self.samples.append(Sample(self,*infoList))

    self.tableName = '_Sample_component'      # This has to be the table 'prefix' before the . as it appears in the NMR-STAR file
    tagNames = ('ID','Mol_common_name','Isotopic_labeling','Assembly_ID','Entity_ID','Concentration_val','Concentration_val_min','Concentration_val_max','Concentration_val_units','Concentration_val_err')
    if self.tableName in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName].tags

      for i in range(0,len(tableTags['Mol_common_name'])):    # Just pick any obligatory tag from the table to loop over the rows

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.sampleComponents.append(SampleComponent(self,*infoList)) # So self is the 'parent' of the SampleCondition data class


class Sample:

  def __init__(self,parent,name,sampleState,details,solventSys,aggrSamNo):

    # Warning: need to do some retyping here, data type not always correct in original NMR-STAR

    self.parent = parent
    self.name = name

    self.sampleState = sampleState
    self.details = details
    self.solventSys = solventSys
    self.aggrSamNo = aggrSamNo

class SampleComponent:

  def __init__(self,parent,Id,name,label,molSysId,molId,conc,conc_min,conc_max,conc_unit,conc_err):

    self.parent = parent
    self.Id = Id
    self.name = name
    self.label = label
    self.molSysId = molSysId
    self.molId = molId

    self.conc = conc
    self.conc_min = conc_min
    self.conc_max = conc_max
    self.conc_unit = conc_unit
    self.conc_err = conc_err


#
# NMR experiment information
#

class NmrStarExperimentFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.experiments = []
    self.experimentListId = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.experimentListId = self.saveFrame.tags['ID']

    tagNameDict = {'Details': 'experimentListDetails'}

    for tagName in tagNameDict.keys():
      if tagName in self.saveFrame.tags:
        setattr(self, tagNameDict[tagName], self.saveFrame.tags[tagName])
      else:
        setattr(self, tagNameDict[tagName], None)

    self.tableName = '_Experiment'

    tagNames = ('ID','Name','Raw_data_flag','Sample_ID','Sample_label','Sample_state','Sample_condition_list_ID','Sample_condition_list_label','NMR_spectrometer_ID','NMR_spectrometer_label','NMR_spectrometer_probe_ID','NMR_spectrometer_probe_label')
    if self.tableName in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName].tags

      for i in range(0,len(tableTags['ID'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.experiments.append(Experiment(self,*infoList))

class Experiment:

  def __init__(self,parent,Id,name,raw_data_flag,sampleId,sampleLabel,sampleState,sampleCondListId,sampleCondListLabel,specId,specLabel,probeId,probeLabel):

    self.parent = parent
    self.Id = Id
    self.name = name
    self.raw_data_flag = raw_data_flag
    self.sampleId = sampleId
    self.sampleLabel = sampleLabel
    self.sampleState = sampleState
    self.sampleCondListId = sampleCondListId
    self.sampleCondListLabel = sampleCondListLabel
    self.specId = specId
    self.specLabel = specLabel
    self.probeId = probeId
    self.probeLabel = probeLabel

#
# NMR spectrometer information
#

class NmrStarInstrumentFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.instruments = []
    self.specId = None
    self.specListDetails = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.specId = self.saveFrame.tags['ID']

    if 'Details' in self.saveFrame.tags:
      self.specListDetails = self.saveFrame.tags['Details']

    self.tableName = '_NMR_spectrometer_view'

    tagNames = ('ID','Name','Manufacturer','Model','Field_strength','Serial_number','Details')
    if self.tableName in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName].tags

      for i in range(0,len(tableTags['ID'])):

        infoList = []
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.instruments.append(NmrSpectrometer(self,*infoList))

class NmrSpectrometer:

  def __init__(self,parent,Id,name,manufacturer,model,field_strength,serialNumber,details):

    self.parent = parent
    self.Id = Id
    self.name = name
    self.manufacturer = manufacturer
    self.model = model
    self.field_strength = field_strength
    self.serialNumber = serialNumber
    self.details = details

#
# NMR Probe information
#

class NmrStarProbeFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.probes = []
    self.probeId = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.probeId = self.saveFrame.tags['ID']

    infoList = []

    tagNames = ('ID','Manufacturer','Model','Serial_number','Details','Diameter')

    for tagName in tagNames:
      if not tagName in self.saveFrame.tags:
        self.saveFrame.tags[tagName] = None
      infoList.append(self.saveFrame.tags[tagName])

    self.tableName = '_NMR_probe'

    if self.tableName in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName].tags

      if 'Type' in tableTags:
        infoList.append(tableTags['Type'][0])
      else:
        infoList.append(None)

    else:
      infoList.append(None)

    self.probes.append(NmrProbe(self,*infoList))

class NmrProbe:

  def __init__(self,parent,Id,manufacturer,model,serialNumber,details,diameter,pType):

    self.parent = parent
    self.Id = Id
    self.manufacturer = manufacturer
    self.model = model
    self.serialNumber = serialNumber
    self.details = details
    self.diameter = diameter
    self.pType = pType

#
# Software information
#

class NmrStarSoftwareFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.software = []
    self.softwareVendors = []
    self.softwareId = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.softwareId = self.saveFrame.tags['ID']

    infoList = []

    tagNames = ('Name','Version','Details')

    for tagName in tagNames:
      if not tagName in self.saveFrame.tags:
        self.saveFrame.tags[tagName] = None
      infoList.append(self.saveFrame.tags[tagName])

    self.tableName = '_Vendor'

    tagNames = ('Name','Address','Electronic_address')
    if self.tableName in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName].tags

      for tagName in tagNames:
        if tagName not in tableTags:
          infoList.append(None)
        else:
          infoList.append(tableTags[tagName][0])

    else:
      infoList.append(None)
      infoList.append(None)
      infoList.append(None)

    self.tableName = '_Task'

    taskString = None

    tagNames = ('Task',)
    if self.tableName in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName].tags

      taskList = []
      for i in range(0,len(tableTags['Task'])):

        for tagName in tagNames:
          if tagName not in tableTags:
            taskList.append(None)
          else:
            taskList.append(tableTags[tagName][i])

        taskString = ', '.join(taskList)

    infoList.append(taskString)

    self.software.append(Software(self,*infoList))

class Software:

  def __init__(self,parent,name,version,details,venName,venAddress,venEAddress,tasks):

    self.parent = parent
    self.name = name
    self.version = version
    self.details = details

    self.venName = venName
    self.venAddress = venAddress
    self.venEAddress = venEAddress
    self.tasks = tasks

#
# Method information
#

class NmrStarMethodFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.method = []
    self.methodId = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.methodId = self.saveFrame.tags['ID']

    infoList = []

    tagNames = ('Details',)

    for tagName in tagNames:
      if not tagName in self.saveFrame.tags:
        self.saveFrame.tags[tagName] = None
      infoList.append(self.saveFrame.tags[tagName])

    self.method.append(Method(self,*infoList))

class Method:

  def __init__(self,parent,details):

    self.parent = parent
    self.details = details

#
# Ensemble stats information
#

class NmrStarConfStatFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame):

    self.confStat = []
    self.confStatId = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.confStatId = self.saveFrame.tags['ID']

    infoList = []

    tagNames = ('Conf_family_coord_set_ID,','Details','Conformer_calculated_total_num','Conformer_submitted_total_num',
                'Conformer_selection_criteria','Representative_conformer','Rep_conformer_selection_criteria')

    for tagName in tagNames:
      if not tagName in self.saveFrame.tags:
        self.saveFrame.tags[tagName] = None
      infoList.append(self.saveFrame.tags[tagName])

    self.confStat.append(ConfStat(self,*infoList))

class ConfStat:

  def __init__(self,parent,ensId,details,calculated,submitted,criteria,representative,repr_criteria):

    self.parent = parent
    self.ensId = ensId
    self.details = details
    self.calculated = calculated
    self.submitted = submitted
    self.criteria = criteria
    self.representative = representative
    self.repr_criteria = repr_criteria

#
# Natural/experimental information
#

class NmrStarSourceFile(NmrStarProjectDataComponent):

  def initialize(self,parent,saveFrame,saveFrameName):

    self.saveFrameName = saveFrameName

    if self.saveFrameName == 'natural_source':
      self.source = 'natural'
    elif self.saveFrameName == 'experimental_source':
      self.source = 'experimental'
    else:
      self.source = None

    self.sources = []
    self.sourceId = None

    self.genericInit(parent,saveFrame)

  def parseSaveFrame(self):

    if not self.checkVersion():
      return

    self.sourceId = self.saveFrame.tags['ID']

    if self.source == 'natural':
      self.tableName = '_Entity_natural_src'
      tagNames = ('ID','Entity_ID','Entity_chimera_segment_ID','Type',None,'NCBI_taxonomy_ID','Organism_name_scientific','Organism_name_common','Strain','Variant','Organ','Tissue','Cell_line','Cell_type','ATCC_number',None,'Plasmid','Details','Species','Genus','Superkingdom','Kingdom',None,'Gene_mnemonic','Organelle')

    elif self.source == 'experimental':
      self.tableName = '_Entity_experimental_src'
      tagNames = ('ID','Entity_ID','Entity_chimera_segment_ID',None,'Production_method','Host_org_NCBI_taxonomy_ID','Host_org_scientific_name','Host_org_name_common','Host_org_strain','Host_org_variant','Host_org_organ','Host_org_tissue','Host_org_cell_line','Host_org_cell_type','Host_org_ATCC_number','Vector_type','Vector_name','Details','Host_org_species','Host_org_genus',None,None,'PDBview_host_org_vector_name',None,'Host_org_organelle')

    if self.tableName in self.saveFrame.tables:
      tableTags = self.saveFrame.tables[self.tableName].tags

      for i in range(0,len(tableTags['ID'])):

        infoList = [self.source]
        for tagName in tagNames:
          if tagName not in tableTags:
            infoList.append(None)
          else:
            infoList.append(tableTags[tagName][i])

        self.sources.append(Source(self,*infoList))

class Source:

  def __init__(self,parent,sourceType,Id,entId,entChimId,orgType,prodMethod,ncbiTaxId,orgSci,orgName,strain,variant,organ,tissue,cell_line,cell_type,atccNo,vecType,plasmid,details,species,genus,superKingdom,kingdom,pdbVectorName,geneMnemonic,organelle):

    self.parent = parent
    self.sourceType = sourceType
    self.Id = Id
    self.entId = entId
    self.entChimId = entChimId
    self.orgType = orgType
    self.prodMethod = prodMethod
    self.ncbiTaxId = ncbiTaxId
    self.orgSci = orgSci
    self.orgName = orgName
    self.strain = strain
    self.variant = variant
    self.organ = organ
    self.tissue = tissue
    self.cell_line = cell_line
    self.cell_type = cell_type
    self.atccNo = atccNo
    self.vecType = vecType
    self.plasmid = plasmid
    self.details = details
    self.species = species
    self.genus = genus
    self.superKingdom = superKingdom
    self.kingdom = kingdom
    self.pdbVectorName = pdbVectorName
    self.geneMnemonic = geneMnemonic
    self.organelle = organelle



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  parser.py
#
#  Copyright 2017  <@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
from b2bTools.general.Io import B2bIo

aa3To1Code={'CYS': 'C', 'ASP': 'D', 'SER': 'S', 'GLN': 'Q', 'LYS': 'K', 'ASN': 'N', 'PRO': 'P', 'THR': 'T', 'PHE': 'F', 'ALA': 'A', 'HIS': 'H', 'GLY': 'G', 'ILE': 'I', 'LEU': 'L', 'ARG': 'R', 'TRP': 'W', 'VAL': 'V', 'GLU': 'E', 'TYR': 'Y', 'MET': 'M'}

ss_3class={'H':'H','G':'H','I':'H','E':'E','B':'E','S':'C','T':'C','C':'C','-':'-','b':'-','R':'-','D':'-','_':'-','A':'-','L':'-'}


class protein_class():
	def __init__(self,chainCode):
		self.chainCode = chainCode
		self.seq=''
		self.resi=[]
		self.ss=''
		self.rci=[]
		self.resnum=[]

class residue_class():
	def __init__(self):
		self.resname=None
		self.atom={}
		self.ss=None
		self.rci=None

def parse_official(fileName,is_star=False,original_numbering=True):

	b2bIo = B2bIo()

	proteins = []

	if type(is_star) == type(""):
		is_star = eval(is_star)

	if is_star:
		allSeqInfo = b2bIo.readNmrStarSequenceShifts(fileName,original_numbering=original_numbering)
	else:
		allSeqInfo = b2bIo.readNefFileSequenceShifts(fileName)

	chainCodes = allSeqInfo.keys()

	for chainCode in chainCodes:

		prot=protein_class(chainCode)

		seqInfo = allSeqInfo[chainCode]

		for (seqElement,shiftInfo) in seqInfo:

			if seqElement['residue_name'] in aa3To1Code:
				resitype=aa3To1Code[seqElement['residue_name']]
			else:
				resitype='X'

			prot.seq += resitype

			resi = residue_class()
			resi.resname = resitype
			prot.resnum += [int(seqElement['sequence_code'])]
			prot.resi += [resi]

			for shiftValue in shiftInfo:
				atom_name=shiftValue['atom_name']
				shift=shiftValue['value']

				resi.atom[atom_name]=shift

		proteins.append(prot)

	return proteins

if __name__ == '__main__':
	a=parse_official('../../test/input/NEF_example.nef')
	print(a)



"""
Classes for dealing with STAR syntax
"""

__author__    = "$Author: jurgenfd $"
___revision__ = "$Revision: 1.6.4.1 $"
___date__     = "$Date: 2011-04-06 09:00:38 $"

"""
$Log: not supported by cvs2svn $
Revision 1.6  2007/07/09 12:22:27  wimvranken
Merge with branch 4.

Revision 1.5.4.1.4.1  2007/06/13 08:35:52  wimvranken
Removed copyright - is not CCPN

Revision 1.5.4.1  2006/07/21 17:29:01  wimvranken
Now only contain full path imports

Revision 1.5  2005/06/27 16:43:42  wb104
Updated licenses.

Revision 1.4  2004/12/15 17:57:20  tjs23
TJS: Updated licenses.

Revision 1.3  2003/08/08 12:46:08  wfv20
Added comment writing support

Revision 1.2  2003/07/14 09:34:32  wfv20
Modified so list references are not carried through class initialization

Revision 1.1  2003/07/01 12:56:18  wfv20
Jurgen Doreleijers modified Python nmrStar reader. Bug fixes and additions (notably for reading nmrView star files) have been made.

Revision 1.1.1.1  2001/11/02 20:16:40  jurgen
Initial package capable of read/write access to STAR files without nested loops

"""

from .Utils import Lister

"""
Saveframe class
"""
class SaveFrame (Lister):
    def __init__( self,
                  title     = 'general_sf_title',
                  tagtables = None,
                  text      = '',
                  verbosity = 2,
                  comment = ''):
        self.title      = title

        # Modified tagtables initialization so list references
        # are not carried through (Wim 14/07/2002)
        self.tagtables = tagtables

        if self.tagtables == None:
          self.tagtables  = []

        self.text       = text
        self.verbosity  = verbosity
        self.comment = comment          # Comment attribute added to node (Wim 2003/08/05)

    "Returns the STAR text representation"
    def star_text (self,
                   flavor = 'NMR-STAR'
                   ):
        str = "\n"
        str = str + 'save_%s\n' % self.title

        for tagtable in self.tagtables:
            str = str + tagtable.star_text( flavor=flavor )

        str = str + '\nsave_\n'
        return str

    "Simple checks on integrity"
    def check_integrity( self,  recursive = 1  ):
        if recursive:
            for tagtable in self.tagtables:
                if tagtable.check_integrity():
                    print("ERROR: integrity check failed for tagtable")
                    return 1
        if self.verbosity >= 9:
            print('Checked integrity of SaveFrame(%2s tagtables, recurs.=%s)  : OK [%s]' % (
                len(self.tagtables), recursive, self.title ))


###############################################################################
if __name__ == '__main__':
    if 1:
        sf = SaveFrame( verbosity=9 )
        sf.check_integrity()
        print("sf:", sf.star_text())
        print("sf:", sf)
        print("FINISHED")



from b2bTools.general.Io import B2bIo


class NMRStarIO:
    @staticmethod
    def read_nmr_star_project(fileName):
        """
        :param filenNme:  Input NMR-STAR file path
        :return: file content connected to nmrStarFile object
        """

        return B2bIo().readNmrStarProject(fileName)

    @staticmethod
    def read_nmr_star_sequence_shifts(fileName, original_numbering=True):
        """
        :param fileName: Input NMR-STAR file path
        :param original_numbering: If set to True (Boolean), will retain original sequence code numbering
        :return: dictionary with sequence and shift information
        """

        return B2bIo().readNmrStarSequenceShifts(fileName, original_numbering)


#!/usr/bin/env python
# encoding: utf-8
"""
Predictor.py

Created by Wim Vranken on 25th July 2018, based on original DynaMine code (Elisa Cilia)
Copyright (c) 2018 Wim Vranken. All rights reserved.
"""

import os
import glob
import numpy

from b2bTools.general import Util as b2bUtil
from b2bTools.general.Io import B2bIo


# TODO: include file/JSON writing class in here for default file output options
class DynaMine(B2bIo):

  version = '4.0'
  # scriptName = "b2bTools.singleSeq.DynaMine.Predictor"
  scriptName = "python.b2bTools.singleSeq.DynaMine.Predictor"
  name = "DynaMine suite"

  aas = '-ACDEFGHIKLMNPQRSTVWYX'

  model_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "models/")

  def __init__(self,predictionTypes=('backbone', 'sidechain', 'ppII', 'coil', 'sheet', 'helix')):

    """
    :param predictionTypes: a list of the prediction types that need to be performed. By default all possible types are done.
    """

    self.windowsize = 51

    self.modelDict = {}
    self.modelBias = {}
    self.input = False
    self.outputfilenames = []
    self.allPredictions = {}

    self.types_ = []
    for predictionType in predictionTypes:
      if predictionType in ('backbone','sidechain'):
        suffix = ''
      else:
        suffix = '_new'
      self.types_.append((predictionType,suffix))

    self.loadModels()
    self.adjustModels()

    # Additional info for writing files
    self.references = ['doi: 10.1038/ncomms3741 (2013)', 'doi: 10.1093/nar/gku270 (2014)']
    self.infoTexts = ['Generated by the DynaMine suite','See http://bio2byte.be']

    self.informationPerPredictor = {
      self.name: {
          'references': self.references,
          'info':    ";".join(self.infoTexts),
          'version': self.version,
          'origin':  self.scriptName
                }
          }

  def loadModels(self):
    modelLoaded = True
    for (type_,suffix) in self.types_:

      pickleWindowList = []
      modelDir = os.path.join(self.model_path, type_)
      pickleFileNames = glob.glob(os.path.join(modelDir,"read*.model{}.pkl.bz2".format(suffix)))
      for pfn in pickleFileNames:
        pickleWindowList.append((int(os.path.split(pfn)[1].split('.')[0][4:]),pfn))

      ### Doesn't find pklfile
      for (w,pklfile) in pickleWindowList:
        #print w, pklfile
        if os.path.isfile(pklfile):
          p = b2bUtil.Pickle()
          model = p.load_data(pklfile)
          modelKey = (type_, w)
          self.modelDict[modelKey] = []
          self.modelBias[modelKey] = model['bias']
          # Precalculate for faster retrieval
          for predPos in list(model.keys()):
            if not predPos == 'bias':
              self.modelDict[modelKey].append({})
              for aa in self.aas:
                self.modelDict[modelKey][-1][aa] = 0.0
              for (correction, aaCodes) in model[predPos]:
                for aaCode in aaCodes:
                  self.modelDict[modelKey][-1][aaCode] += correction
      else:
        modelLoaded = False
    return modelLoaded

  def adjustModels(self):
    """ Substitute the values for '-' with the median over the position
    """
    for modelKey, model in self.modelDict.items():
      # w = len(model) / 2
      for i in range(len(model)):
        newdistribution = self.filterOutliers(list(model[i].values())) # remove outliers
        #newdistribution = model[i].values()
        m = numpy.median(newdistribution) # compute the median per position
        # print "before:", model[i]['-'], "after:", m
        model[i]['-'] = m

  # Warning: THIS IS NOW NOT WORKING FROM A DICT BUT FROM A LIST OF TUPLES!
  def predictSeqs(self, seqs, force_windowsize=None):

    """
    :param seqs: List/tuple of (seqId,sequenceString) tuples
    :param force_windowsize: Force a particular sequence window size when running
    :return: True if all sequences predicted, False if problems

    See self.allPredictions for the predictions themselves.
    """

    count_short = 0

    for (seqId,sequence) in seqs:

      print('Predicting {}...\n'.format(seqId), end=' ')

      seqLen = len(sequence)
      self.allPredictions[seqId] = {}

      if force_windowsize:
        self.windowsize = force_windowsize
      else:
        if seqLen > 51:
          self.windowsize = 51
        elif seqLen >= 25:
          self.windowsize = 25
        elif seqLen >= 19:
          self.windowsize = 19
        elif seqLen == 5:
          self.windowsize = 5
        elif seqLen < 5:
          self.windowsize = 5
          count_short += 1
          print('Skipping, too short.')
          continue
        else:
          if seqLen % 2 == 0:
            self.windowsize = seqLen - 1
          else:
            self.windowsize = seqLen

      for (predictionType,suffix) in self.types_:
        self.allPredictions[seqId][predictionType] = self.computePredictions(sequence, predictionType)

    if count_short == len(seqs):
      return False
    else:
      return True

  def computePredictions(self, s, type_):
    """
    Returns a list of lists (one for each amino acid in the sequence) containing two strings ['aa type one-letter code', 'prediction']
    """
    # Wim speedup attempt
    emptyTailLen = int((self.windowsize - 1) / 2)
    s_app = self.correctFragment("-" * emptyTailLen + "".join([x[0] for x in s]) + "-" * emptyTailLen)
    #print s_app
    modelKey = (type_, self.windowsize)
    seqLen = len(s_app)
    preds = []

    for seqPos in range(seqLen):
      preds.append(self.modelBias[modelKey])

    for seqPos in range(seqLen):
      aaCode = s_app[seqPos]
      predPosBase = emptyTailLen + seqPos
      for otherSeqPos in range(max(0,seqPos - emptyTailLen),min(seqLen,seqPos+emptyTailLen+1)):
        predPos = predPosBase - otherSeqPos
        preds[otherSeqPos] += self.modelDict[modelKey][predPos][aaCode]

    for seqPos in range(seqLen):
      preds[seqPos] = (s_app[seqPos],preds[seqPos])

    return preds[emptyTailLen:-emptyTailLen]

  #
  # Functions ripped from other scripts in original dynaMine distr
  #

  def correctFragment(self,frag):
    seq = ''
    for j in frag:
      if not j[0] in self.aas:
        seq += 'X'
      else:
        seq += j[0]
    return seq

  def filterOutliers(self,values):
    q3 = numpy.percentile(values, 75, interpolation='higher')
    q1 = numpy.percentile(values, 25, interpolation='lower')
    iqr = q3 - q1
    newdistribution = [v for v in values if v >= q1 - 1.5 * iqr and v <= q3 + 1.5 * iqr]
    return newdistribution



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  autoenc.py
#  
#  Copyright 2017  <@gmail.com>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  
import torch
import torch.autograd as autograd
import torch.nn as nn

import numpy as np
torch.manual_seed(1)
EPOCH=10
class nn_class_autoenc(nn.Module):
		def __init__(self, n_features,hidden_dim=10,cuda=False):
			super(nn_class_autoenc, self).__init__()

			self.encoder = nn.Sequential(
				nn.Linear(n_features, hidden_dim),
				nn.ReLU(),
				nn.Linear(hidden_dim, hidden_dim),
				nn.ReLU(),
				#nn.Linear(hidden_dim, hidden_dim//2),
				#nn.Tanh(),
				nn.Linear( hidden_dim, 1),
				nn.Sigmoid()
				)
			self.decoder = nn.Sequential(
				nn.Linear( 1,hidden_dim),
				nn.ReLU(),
				nn.Linear(hidden_dim, hidden_dim),
				nn.ReLU(),
				nn.Linear(hidden_dim,n_features),
				nn.Sigmoid()
			)

		def forward(self, x):
				encoded = self.encoder(x)
				decoded = self.decoder(encoded)
				return encoded, decoded
		def encode_x(self,x):
			return self.encoder(x)
		def decode_x(self,x):
			return self.decoder(x)
			
class autoencoder():
	def __init__(self, hidden_dim=100,cuda=False):
		self.hidden_dim=hidden_dim
		self.cuda=cuda
	def fit(self,x,batch=100,EPOCH=10,LR=0.01):
		nn_auto=nn_class_autoenc(len(x[0]),hidden_dim=self.hidden_dim)
		optimizer = torch.optim.Adam(nn_auto.parameters())
		loss_func = nn.MSELoss()
		#plt.ion()
		#fig, ax = plt.subplots(nrows=1, ncols=1)
		for epoch in range(EPOCH):
			decoded_all=[]
			x_all=[]
			for i in range(0,len(x),batch):
				if i+batch>len(x):
					end=len(x)
				else:
					end=len(x)+i
				if self.cuda:
					b_x = autograd.Variable(torch.Tensor(x[i:end])).cuda() 
					b_y = autograd.Variable(torch.Tensor(x[i:end])).cuda() 
				else:
					b_x = autograd.Variable(torch.Tensor(x[i:end])) 
					b_y = autograd.Variable(torch.Tensor(x[i:end]))

				encoded, decoded = nn_auto(b_x)
				decoded_all+=list(decoded.data.numpy())
				x_all+=list(x[i:end])
				loss = loss_func(decoded[:,:4], b_y[:,:4])     
				optimizer.zero_grad()              
				loss.backward()

				

				#ax.plot([np.array(x_all)[:,0],np.array(x_all)[:,1]],'ro')
				
				
				optimizer.step() 
			print(epoch,loss.data.numpy()[0]) 
			'''
			ax.clear()
			a=np.array(decoded_all)[0:100,0]
			b=np.array(decoded_all)[0:100,1]

			#assert len(a)==len(b)
			
			ax.scatter(a,b,alpha=1)
			ax.scatter(np.array(x_all)[:,0],np.array(x_all)[:,1],color='r',alpha=0.01)  
			plt.draw(); plt.pause(0.05)
			print loss.data  

			ax.clear()
			ax.plot([np.array(decoded_all)[0:100,0],np.array(decoded_all)[0:100,1]],'bo')
			ax.plot([np.array(x_all)[:,0],np.array(x_all)[:,1]],'ro')
			plt.draw(); plt.pause(0.05)
			'''
		self.model=  nn_auto  
	def encode(self,x):

		if self.cuda:
			b_x = autograd.Variable(torch.Tensor(x)).cuda() 
			b_y = autograd.Variable(torch.Tensor(x)).cuda() 
		else:
			b_x = autograd.Variable(torch.Tensor(x)) 
			b_y = autograd.Variable(torch.Tensor(x)) 
		y_b=self.model.encode_x(b_x)
		#print len(y_b)
		return y_b.data.numpy().flatten()
	def decode(self,x):

		if self.cuda:
			b_x = autograd.Variable(torch.Tensor(x)).cuda() 
			b_y = autograd.Variable(torch.Tensor(x)).cuda() 
		else:
			b_x = autograd.Variable(torch.Tensor(x)) 
			b_y = autograd.Variable(torch.Tensor(x)) 
		y_b=self.model.decode_x(b_x)
		
		return y_b.data.numpy()
def main(args):
	x=np.random.rand(100,3)
	a=autoencoder()
	a.fit(x)
	v= a.encode(x)
	print(a.decode(v.reshape(len(v),1)))

if __name__ == '__main__':
		import sys
		sys.exit(main(sys.argv))



from fileinput import filename
from b2bTools.general.Io import B2bIo


class FastaIO:

	@staticmethod
	def read_fasta_from_file(fileName):

		"""
		Reads a FASTA file

		:param fileName: Path to the FASTA file
		:return: List of (seqId, sequenceString) tuples
		"""

		return B2bIo().readFasta(fileName=fileName)

	def read_fasta_from_string(text):
		"""
		Reads a FASTA from a text string

		:param text: Text in the FASTA format
		:return: List of (seqId, sequenceString) tuples
		"""
		return B2bIo().readFasta(None, fileString=text)

	@staticmethod
	def write_fasta(fastaFileName, sequences):

		"""
		Writes sequences in the FASTA format

		:param fastaFileName: Path to the FASTA file
		:param sequences: List of (seqId, sequenceString) tuples
		:return None
		"""

		return B2bIo().writeFasta(fastaFileName=fastaFileName, sequences=sequences)
